package graphic;

import javax.swing.*;
import java.awt.*;
import java.util.*;

import server.event.*;

/**
 * The console prints out error-messages. It can be called forward by pressing CTRL+ALT+E.
 * The Console is a singleton.
 */
class ErrorConsole extends MemoryFrame implements Observer{

	private static ErrorConsole ec;
	private JTextArea paper = new JTextArea();
	

	/**
	 * This private constructor constructs the graphics.
	 *
	 */
	private ErrorConsole() {
		super("ErrorConsole");
		
		setSize(500,300);
		restore();
		
		getContentPane().setLayout(new GridLayout(1,1));
		paper.setEditable(false);
		addText("Denne konsol udskriver fejlmeddelelser");
		addText("Disse kan mailes til: Kenneth@jqdata.dk. P forhnd tak.");
		addText("_____________________________________________________________________");
		getContentPane().add(new JScrollPane(paper));		
		BigBrother.getInstance().addObserver(this);
	}
	
	public void setText() {
	}

	/**
	 * Adds the string, and a carrige return.
	 *
	 * @param   text  
	 */
	public void addText(String text) {
		paper.append(text + "\n");
	}
	

	/**
	 * Prints the argument in the Errorconsole. If the argument is an Exception, 
	 * it is printed with a stacktrace.
	 *
	 * @param   o  ...
	 * @param   arg  An Exception, or a text
	 */
	public void update(Observable o, Object arg) {
		if (arg instanceof Exception) {
			addText(arg.toString());
			StackTraceElement[] el = ((Exception)arg).getStackTrace();
			for (int i = 0 ; i < el.length ; i++)
				addText("      at "+el[i]);
		}else
			addText(arg.toString());
	}
	

	/**
	 * Returns an instance...
	 *
	 * @return     instance.
	 */
	public static ErrorConsole instanceOf() {
		if (ec == null)
			ec = new ErrorConsole();
		return ec;	
	}
}