package graphic;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

import server.*;
import setup.*;
import graphic.services.*;

public class SetupBan extends JPanel implements ActionListener{
	
	private Setup setup;
	private JButton ny, slet, opdater, gem;
	private JLabel text;
	private JQTable table = new JQTable(new String[]{""});
	private ResourceBundle res;
	
	public SetupBan(Setup setup) {
		this.setup = setup;
		this.res = res;
		GridBagGUI gbg = new GridBagGUI(this);
		gbg.anchor("NW");
		gbg.size(5,1);
		gbg.fill("H");
		gbg.weighty(1);
		gbg.position(0,0);
		text = new JLabel();
		gbg.add(text);
		
		JScrollPane pane = new JScrollPane(table);
		pane.getViewport().setBackground(new Color(255,255,255));
		pane.setPreferredSize(new Dimension(200,200));
		gbg.position(0,1);
		gbg.add(pane);
		
		ny = new JButton();
		ny.addActionListener(this);
		gbg.size(1,1);
		gbg.anchor("SE");
		gbg.fill("N");
		gbg.weighty(0);
		gbg.position(1,2);
		gbg.add(ny);
		
		slet = new JButton();
		slet.addActionListener(this);
		gbg.position(2,2);
		gbg.add(slet);
		
		opdater = new JButton();
		opdater.addActionListener(this);
		gbg.position(3,2);
		gbg.add(opdater);
		
		gem = new JButton();
		gem.addActionListener(this);
		gbg.position(4,2);
		gbg.add(gem);
		restore();
	}
	
	public void setText(ResourceBundle res) {
		//.setText(res.getString(""));
		table.getColumnModel().getColumn(0).setHeaderValue(res.getString("IP - Address"));
		text.setText(res.getString("<html>Banned IP addresses are not allowed access to the server.<br>The connection to them are closed immediate <br>after connection and positive verification of the IP address! <html>"));
		ny.setText(res.getString("New"));
		slet.setText(res.getString("Delete"));
		opdater.setText(res.getString("Update"));
		gem.setText(res.getString("Save"));
	}
	
	public void restore() {
		//Fyld tabel med IP Adresser
		table.reset();
		XMLTag xml = Conf.getRoot();
		
		String ban="";
		try{
      ban = xml.getQuickOptionValue("Server.banned");
      StringTokenizer str = new StringTokenizer(ban, ",");
      while (str.hasMoreTokens()) {
    		table.add(new String[]  {str.nextToken().trim()} );
      }	
    }catch(NoSuchTagException e){
		}
	}
	
	public void save() {
		String ip="";
		
		int rows = table.getRowCount();
		String[] arr;
		
		if (rows >= 1) {
			arr = (String[]) table.getRow(0);
			ip = arr[0];
		}
		
		for (int i = 1 ; i < rows; i++) {
			arr = (String[]) table.getRow(i);
			ip+= ", "+arr[0];
		}
		
		try {
			Conf.getRoot().setQuickOptionValue("Server.banned", ip);
		}catch(NoSuchTagException nste) {
		}
	}
	
	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
		
		if (source == opdater) {
			restore();
		}else if(source == gem) {
			setup.saveInfo();
		}else if(source == slet) {
			int i = table.getSelectedRow();
			if (i >= 0)
				table.removeRow(i);
		}else if(source == ny) {
			String tmp = JOptionPane.showInputDialog(this,"Indtast IP - Adresse (f.eks. 127.0.0.1)", "Indtast IP - Adresse", JOptionPane.QUESTION_MESSAGE);
			
			//Valider IP adressen
			StringTokenizer st = new StringTokenizer(tmp,".");
			if (tmp.length() != 0 && st.countTokens() != 4) {
				say("IP adressen ["+tmp+"] er ikke valid!");
			}else if (tmp.length() != 0){
				int tal;
				String token;
				boolean ok = true;
				for (int i = 0 ; st.hasMoreTokens() ; i++ ) {
					token = st.nextToken();
					try {
						tal = Integer.parseInt(token);
						if (!(tal <= 255 && tal >= 0) || token.length() > 3) {
							say("IP adressen ["+tmp+"] er ikke valid. Alle tal skal vre mellem 0 og 255! Fejlen var: ["+token+"].");
							ok = false;
						}
					}catch(NumberFormatException nfe) {
						say("IP adressen ["+tmp+"] er ikke valid! Fejlen var: ["+token+"].");
						ok = false;
					}
				}
				
				if (ok) {
					Object[] o;
					for (int i = table.getRowCount()-1 ; i >= 0 ; i--) {
						o = table.getRow(i);
						if (o[0].toString().equals(tmp)) {
							ok = false;
							break;
						}
					}
				}
				
				if (ok) {
					table.add(new String[]  {tmp});
				}
			}
		}
	}
	
	private void say(String comment) {
		JOptionPane.showMessageDialog(this, comment, "Advarsel!" ,JOptionPane.WARNING_MESSAGE); 
	}
}