package graphic;

import javax.swing.*;
import javax.swing.table.*;
import java.awt.*;
import java.text.*;
import java.awt.event.*;
import java.util.*;

import graphic.services.*;

import setup.*;
import server.*;
import server.event.*;

public class UserList extends MemoryFrame implements ActionListener, ServerObserver{

	private JComboBox users;
	private JRadioButton log, online;
	private ButtonGroup group;
	private JButton ok,vis,close;
	private JToggleButton realTime;
	private JQTable table;
	private boolean showingOnline;
	private Server server = Main.server;	
	private HashMap map = Main.map;
	private JScrollPane pane;
	private ClientObserver observer = new ClientObserver();	
	
	public UserList(String login, boolean showOnline) {
		super("","Session list");
		this.showingOnline = !showOnline; //pga. metode til at undg dobbeltklick
		setIconImage(ImageHandler.getImage("sessions.gif"));
		Main.server.addServerObserver(this);
		draw();
		
		XMLTag[] xml = Conf.getUsers();
		//dirs = new Dir[xml.length];
		users.addItem("Alle");
		Object mem = users.getItemAt(0);
		for (int i = 0 ; i < xml.length ; i++) {
			User u = new User(xml[i]);
			users.addItem(u);
			if (u.getLogin().equals(login))
				mem = u;
		}
		
		users.setSelectedItem(mem);
		
		if (showOnline)
			online.doClick();
		else
			log.doClick();	
		
		realTime.doClick();	
		setText();//This window isn't instantiated at startup, so it's text must be set!
		setVisible(true);
	}
	
	private void draw() {
		Container c = getContentPane();
		c.setLayout(new BorderLayout());
		
		//Top
		JPanel top = new JPanel();
		GridBagGUI gbg = new GridBagGUI(top);
		
		users = new JComboBox();
		users.setPreferredSize(new Dimension(220,25));
		users.addActionListener(this);
		gbg.anchor("NW");
		gbg.position(0,0);
		gbg.insets(0,0,5,5);
		gbg.add(users);
		
		log = new JRadioButton();
		log.addActionListener(this);
		gbg.position(1,0);
		gbg.add(log);
		
		online = new JRadioButton();
		online.addActionListener(this);
		gbg.position(2,0);
		gbg.weightx(1);
		gbg.add(online);
		
		group = new ButtonGroup();
		group.add(log);
		group.add(online);

		c.add(top,BorderLayout.NORTH);
		
		//Midt
		table = new JQTable(new String[] {});
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.setSortByClickActive(true); 
		
		pane = new JScrollPane(table);
		pane.getViewport().setBackground(new Color(255,255,255));
		pane.setVerticalScrollBarPolicy(pane.VERTICAL_SCROLLBAR_ALWAYS);
		c.add(pane,BorderLayout.CENTER);

		//Bund
		JPanel bund = new JPanel();
		GridBagGUI bgbg = new GridBagGUI(bund);
		
		vis = new JButton();
		vis.addActionListener(this);
		bgbg.anchor("NW");
		bgbg.insets(5,5,5,0);
		bgbg.position(0,0);
		bgbg.add(vis);
		
		realTime = new JToggleButton();
		realTime.addActionListener(this);
		bgbg.position(1,0);
		bgbg.add(realTime);
		
		close = new JButton();
		close.addActionListener(this);
		close.setPreferredSize(new Dimension(130,27));
		bgbg.position(2,0);
		bgbg.add(close);
		
		ok = new JButton();
		ok.addActionListener(this);
		bgbg.anchor("NE");
		bgbg.insets(5,5,5,5);
		bgbg.weightx(1);		
		bgbg.position(3,0);
		bgbg.add(ok);
		
		c.add(bund,BorderLayout.SOUTH);
		
		restore();
	}
	
	public void setText() {
		setTitle(res.getString("Sessionlist"));
		//Sets the selected item reselected after changing the "All" object first in the list.
		int val = users.getSelectedIndex();
		users.insertItemAt(res.getString("All"),0);
		users.removeItemAt(1);
		users.setSelectedIndex(val);
		
		if (showingOnline) {
			close.setText(res.getString("Kick"));
		}else {
			close.setText(res.getString("Delete Log"));
		}
		
		if (showingLogin()) {
			table.getColumnModel().getColumn(0).setHeaderValue(res.getString("Username"));
			table.getColumnModel().getColumn(1).setHeaderValue(res.getString("Time"));
			table.getColumnModel().getColumn(2).setHeaderValue(res.getString("Session"));
			table.getColumnModel().getColumn(3).setHeaderValue(res.getString("IP"));
		}else {
			table.getColumnModel().getColumn(0).setHeaderValue(res.getString("Time"));
			table.getColumnModel().getColumn(1).setHeaderValue(res.getString("Session"));
			table.getColumnModel().getColumn(2).setHeaderValue(res.getString("IP"));
		}
	
		log.setText(res.getString("Log"));
		online.setText(res.getString("Online"));
		vis.setText(res.getString("Show"));
		realTime.setText(res.getString("Update Realtime"));
		ok.setText(res.getString("Close"));
	}
	
	private boolean showingLogin() {
		if (users.getSelectedItem() instanceof User) {
			return false;
		}else
			return true;
	}
	
  public void setTable() {
		boolean showUsers = showingLogin();
		
		String[] arr;
		int[] size;
		if (showUsers) {
			arr = new String[] {"","","","",""};
			size = new int[]  {169,155,50,110,0};
			setSize(510,200);
		}else {
			arr = new String[] {"","","",""};
			size = new int[]  {284,50,110,0};//{194,50,110,0};
			setSize(470,200);//380,200
		}
		
		if (showingOnline) {
			table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		}else {
			table.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		}
		
			
		JQTableData model = new JQTableData(arr);
		model.setSize(size);
		table.setJQTableData(model);
		
		TableColumnModel tcm = table.getColumnModel();
		TableColumn tc = tcm.getColumn(tcm.getColumnCount()-1);
		tcm.removeColumn(tc);
		
		
		if (showingOnline) {
			UserSession[] userArr = server.getOnlineUsers();
			for (int i = 0 ; i < userArr.length ; i++) {
				if (showUsers || userArr[i].getLogin().equals(((User)users.getSelectedItem()).getLogin())) {
					if (showUsers)
						table.add(new Object[] {userArr[i].getLogin(), userArr[i].getLogonTime(), ""+userArr[i].getSessionNr(), userArr[i].getClientIP(),userArr[i]});
					else
						table.add(new Object[] {userArr[i].getLogonTime(), ""+userArr[i].getSessionNr(), userArr[i].getClientIP(),userArr[i]});
				}
			}
		}else {
			LogFileReader[] logArr = server.getLogFiles();
			for (int i = 0 ; i < logArr.length ; i++) {
				if (showUsers || logArr[i].getLogin().equals(((User)users.getSelectedItem()).getLogin())) {
					int[] sesnr = logArr[i].getSessionNumbers();
					for (int j = 0 ; j < sesnr.length ; j++) {
						LogFileSession s = logArr[i].getSession(sesnr[j]);
						if (showUsers)
							table.add(new Object[] {s.getLogin(), s.getLogonTime(), ""+s.getSessionNr(), s.getClientIP(), s});
						else
							table.add(new Object[] {s.getLogonTime(), ""+s.getSessionNr(), s.getClientIP(), s});
					}
				}
			}
		}
		int rows = table.getRowCount()-1;
		if (rows>0) {
			table.setRowSelectionInterval(rows,rows);
			JViewport p = pane.getViewport();
			Point point = p.toViewCoordinates(new Point(0,table.getRowHeight()*rows));
			p.setViewPosition(point);
		}
		setText();
  }
	
	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
		
		if (source == log && showingOnline) {
			showingOnline = false;
			setTable();
		}else if (source == online && !showingOnline) {
			showingOnline = true;
			setTable();
		}else if (source == users) {
			setTable();
		}else if (source == vis) {
			int i = table.getSelectedRow();
			if (i >= 0) {
				JQTableData td = table.getJQTableData();
				Object o = td.getValueAt(i, table.getColumnCount());
				if (o instanceof UserSession) {
					Sniffer snif = (Sniffer) map.get(((UserSession)o).getSessionNr()+"");
					if (snif!=null)
						snif.setVisible(true);
				}else
					new Sniffer((LogFileSession)o);
			}
		}else if(source == realTime) {
			if (realTime.isSelected()) {
				setTable();
			}
		}else if(source == ok) {
			dispose();
		}else if(source == close) {
			boolean showingLogin = showingLogin();
			Object[] obj;
			int place;
			if (showingLogin)
				place=4;
			else
				place=3;
			
			if (showingOnline) {
				UserSession ses;
				if (table.getSelectedRow() != -1) {
					obj = table.getRow(table.getSelectedRow());
					ses = (UserSession) obj[place];
					ses.disconnectClient(UserSessionObserver.USER_KICKED);
				}
			}else {
				int[] rows = table.getSelectedRows();
				LogFileSession[] ses = new LogFileSession[rows.length];	
				for (int i = 0 ; i < ses.length ; i++) {
					 obj = table.getRow(rows[i]);
					 ses[i] = (LogFileSession) obj[place];
				}
				server.deleteLogFileSessions(ses);
				setTable();				
			}
		}
	}
	
	public void clientConnected(UserSession userSession) {
		userSession.addUserSessionObserver(observer);
		if (realTime.isSelected() && showingOnline)
			setTable();
	}

	public void clientDisconnected(UserSession userSession) {
		if (realTime.isSelected())
			setTable();
	}
	
	public void serverStarted(boolean ok) {
	
	}
	
	public void serverStoped() {
	
	}
	
	private class ClientObserver extends UserSessionObserverAdapter {
		public void userLogedIn(UserSession usersession)  {
			if (realTime.isSelected() && showingOnline)
				setTable();
		}
	}
	
	private class User {
	
		public XMLTag xml;
	
		public User(XMLTag xml) {
			this.xml = xml;
		}
		
		public XMLTag getTag() {
			return xml;
		}

		private String get(String name) {
			return Conf.getUserOption(xml,name);
		}
		
		public String getLogin() {
			return get("username");
		}

		public String getUsername() {
			return get("name");
		}
		
  	public String toString() {
			if (getUsername().equals(""))
				return getLogin();
			else	
				return getLogin() + " (" + getUsername() + ")";
		}
	}
}