package graphic.services;

import javax.swing.*;
import java.awt.*;

/**
 * Toolkit to build op GUI.
 */
public class GridBagGUI {
  
  private GridBagConstraints gbc;
  private GridBagLayout layout;

  private Container container;
  
  Font myFont2;

  public GridBagGUI( Container container ) {
    this.container = container;

    layout = new GridBagLayout();
    container.setLayout( layout );
    
    reset();
  }
  
  public void add( Component component ) {
    layout.setConstraints( component, gbc );

    component.setFont(myFont2);
    container.add( component );
  }
  
  /*
   *   Constraints methods
   */
  
  public void reset() {
    gbc = new GridBagConstraints();
    myFont2 = new Font("Times New Roman", 0, 14);
  }
  
  public void setFont(String fontName, int style, int fontSize) {
    myFont2 = new Font( fontName, style , fontSize );
  }

  public void size( int width, int height ) {
    gbc.gridwidth = width;
    gbc.gridheight = height;
  }
  
  public void position( int x, int y ) {
    gbc.gridx = x;
    gbc.gridy = y;
  }
  
  public void fill( int con ) {
    gbc.fill = con;
  }
  
  public void fill( char con ) {
    con = Character.toUpperCase( con );
    
    switch ( con ) {
      case 'H':
        fill( GridBagConstraints.HORIZONTAL );
        break;
                
      case 'V':
        fill( GridBagConstraints.VERTICAL );
        break;
                
      case 'B':
        fill( GridBagConstraints.BOTH );
        break;
                
      case 'N':
        fill( GridBagConstraints.NONE );
        break;
                
      default:
        System.out.println( "[GridBagGUI.fill( '" + con + "' )] Unknown GridBagConstraint mnemonic" );
        System.exit( 0 );
    }
  }
  
  public void fill( String con ) {

    if ( con.length() == 1 )
      fill( con.charAt( 0 ) );
    else {
      con = con.toUpperCase();
      
      if ( con.equals( "HORIZONTAL" ) )
        fill( GridBagConstraints.HORIZONTAL );
      else if ( con.equals( "VERTICAL" ) ) 
        fill( GridBagConstraints.VERTICAL );
      else if ( con.equals( "BOTH" ) ) 
        fill( GridBagConstraints.BOTH );
      else if ( con.equals( "NONE" ) ) 
        fill( GridBagConstraints.NONE );
      else {
        System.out.println( "[GridBagGUI.fill( '" + con + "' )] Unknown GridBagConstraint mnemonic" );
        System.exit( 0 );
      }
    }
  }
  
  public void weightx( double weight ) {
    gbc.weightx = weight;
  }
  
  public void weighty( double weight ) {
    gbc.weighty = weight;
  }
  
  public void insets( int top, int left, int botton, int right ) {
    gbc.insets = new Insets( top, left, botton, right );
  }
  
  public void insets( Insets insets ) {
    gbc.insets = insets;
  }
  
  public void ipad( int padx, int pady ) {
    gbc.ipadx = padx;
    gbc.ipady = pady;
  }
  
  public void anchor( int dir ) {
    gbc.anchor = dir;
  }
  
  public void anchor( String dir ) {
    dir = dir.toUpperCase();
    
    if ( dir.equals( "CENTER" ) || dir.equals( "C" ) )
      anchor( GridBagConstraints.CENTER );
    else if ( dir.equals( "NORTH" ) || dir.equals( "N" ) ) 
      anchor( GridBagConstraints.NORTH );
    else if ( dir.equals( "NORTHEAST" ) || dir.equals( "NE" ) ) 
      anchor( GridBagConstraints.NORTHEAST );
    else if ( dir.equals( "EAST" ) || dir.equals( "E" ) ) 
      anchor( GridBagConstraints.EAST );
    else if ( dir.equals( "SOUTHEAST" ) || dir.equals( "SE" ) ) 
      anchor( GridBagConstraints.SOUTHEAST );
    else if ( dir.equals( "SOUTH" ) || dir.equals( "S" ) ) 
      anchor( GridBagConstraints.SOUTH );
    else if ( dir.equals( "SOUTHWEST" ) || dir.equals( "SW" ) ) 
      anchor( GridBagConstraints.SOUTHWEST );
    else if ( dir.equals( "WEST" ) || dir.equals( "W" ) ) 
      anchor( GridBagConstraints.WEST );
    else if ( dir.equals( "NORTHWEST" ) || dir.equals( "NW" ) ) 
      anchor( GridBagConstraints.NORTHWEST );
    else {
      System.out.println( "[GridBagGUI.anchor( '" + dir + "' )] Unknown GridBagConstraint mnemonic" );
      System.exit( 0 );
    }
  }
}