package graphic.services;

import javax.swing.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;
/**
 * This is an easy way to handle a JTable. It provides services for sorting and sized of columns.
 */

public class JQTable extends JTable {
  
  private String[] name;
  private int[] size;
  private JQTableData tableModel;
  private boolean lock=false;
  private boolean ascendedSorted=false;
  private boolean sorted=false;
  private int columnSorted=-1;
  private ColumnListener columnListener;
	private JQTableSortMethod ts;
  
  /**
   * Constructor
   * 
	 * @param   name  Column names
	 * @param   size  Column sizes
   */
  public JQTable(String[] name, int[] size){
    this.name = name;
    this.size = size;
    setAutoResizeMode(AUTO_RESIZE_OFF);
    start();
  }
  
  /**
   * Constructor
   * 
	 * @param   name  Column names
   */
  public JQTable(String[] name){
    this.name = name;
    this.size = new int[name.length];
    start();
  }
  

  /**
   * Constructor
   *
   * @param   tm TableModel
   */
  public JQTable(JQTableData tm){
    makeTable(tm);
  } 
  
  private void start(){
    tableModel = new JQTableData( name );
    tableModel.setSize(size);
    makeTable(tableModel);
  }
  
  private void makeTable(JQTableData tm){
		tableModel = tm;
		setAutoCreateColumnsFromModel( false );
    setModel( tableModel );
    getTableHeader().setReorderingAllowed( false );
    
    for (int i = 0 ; i<tableModel.getColumnCount();i++ )
      addColumn( new TableColumn(i,tableModel.getSize(i)) );
  }
  
  /**
   * Sets the TableModel for this table.
   *
   * @param   tableModel  TableModel
   */
  public void setJQTableData(JQTableData tableModel){
    size = tableModel.getSize();
    
    if (size == null){
      size = new int[tableModel.getColumnCount()];
    }else
      setJQTableData(tableModel,size);
  }
  
  /**
   * Sets the TableModel for this table, and sets the columns sizes
   *
   * @param   tableModel  TableModel
	 * @param   size  column sizes
   */
  public void setJQTableData(JQTableData tableModel, int[] size){
    reset();
    tableModel.setSize(size);
    
    if (!lock){
      TableColumnModel tcm = getColumnModel();
      while(tcm.getColumnCount() > 0){
        tcm.removeColumn(tcm.getColumn(0));
      }
    }

    this.tableModel = tableModel;
    setModel(tableModel);
    
    if (!lock){
      if (tableModel.getSize(0)!=0)
        setAutoResizeMode(AUTO_RESIZE_OFF);
      else  
        setAutoResizeMode(AUTO_RESIZE_ALL_COLUMNS);
    
      for (int i = 0 ; i<tableModel.getColumnCount();i++ ){
        addColumn( new TableColumn(i,tableModel.getSize(i)) );
      }
    }
  }
  

  /**
   * The classes TableModel
   *
   * @return     TableModel
   */
  public JQTableData getJQTableData(){
    return tableModel;
  }

	/**
	 * Adds a row to the table.
	 *
	 * @param   obj  the row Object[]
	 */

  public void add(Object[] obj){
    tableModel.add(obj);
  }
  
  /**
   * Adds a row to the table on a certain position.
   *
   * @param   obj  the row Object[]
   * @param   place the placement p. 0 = top.
   */
  public void add(Object[] obj, int place){
    tableModel.add(place, obj);
  }

	/**
	 * Resets the table. WARNING!! This will delete the entire table!
	 *
	 */
  public void reset(){
    tableModel.reset();
  }
  

  /**
   * If this lock is on, the columns will not change when setting a new JQTableData.
   *
   * @param   on  locked/unlocked (true/false)
   */
  public void setColumnsLocked( boolean on ){
    lock = on;
  }
  

  /**
   * Is the columns locked?
   *
   * @return     locked/unlocked (true/false)
   */
  public boolean isColumnsLocked(){
    return lock;
  }
  

  /**
   * Gets a row
   *
   * @param   i  the row number
   * @return     the entire row
   */
  public Object[] getRow(int i){
    return (Object[]) tableModel.getRow(i);
  }
  

  /**
   * Removes a row from the table
   *
   * @param   i  The row that should be deleted
   */
  public void removeRow(int i){
    tableModel.remove(i);
  }
  

  /**
   * Replaces a row with a new one
   *
   * @param   i  The row, that should be replaced
   * @param   obj  The new row
   */
  public void replace(int i, Object[] obj){
  	tableModel.replace(i, obj);
  }


  /**
   * Finds the row-number where ID is in the column.
   *
   * @param   ID  The value to look for
   * @param   column  The column to search in
   * @return     The row, that th value was found in.
   */
  public int findRow(int ID, int column){
    for (int i = 0; i< tableModel.getRowCount();i++){
      int id = Integer.parseInt(""+tableModel.getValueAt(i, column));
      if ( ID == id )
        return i;
    }
    return -1;
  }
  
  /**
   * Made of implementation reasons.
   *
   * @param   aColumn  Column for deletion.
   */
  public void removeColumn(TableColumn aColumn){
    tableModel.setVisibleColumn(aColumn.getModelIndex(), false);
    super.removeColumn(aColumn);
  }
  
  /**
   * Sorts the thable by a certain column
   *
   * @param   columnNr  Column Number
   * @param   ascending Ascending(true), Descending(false).
   */
  public void sortByColumnNr(int columnNr, boolean ascending){
    sorted=true;
    ascendedSorted=ascending;
    columnSorted=columnNr;
    tableModel.sortByColumnNr(columnNr, ascending, ts);
  }
  
  /**
   * Sets sorting of column by a click in the Header active. 
   *
   * @param   on  on/off
   */
  public void setSortByClickActive(boolean on){
    if (on && columnListener==null){
      columnListener = new ColumnListener();
      getTableHeader().addMouseListener(columnListener);
    }else if(!on && columnListener!=null){
      getTableHeader().removeMouseListener(columnListener);
      columnListener=null;
    }
  }

  /**
   * If sort by click in header is active or not.
   *
   * @return     on/off
   */
  public boolean isSortByClickActive(){
    if (columnListener!=null)
      return true;
    else
      return false;
  }
  
  
  //Mouselistener. Used when isSortByClickActive() returns true.
  class ColumnListener extends MouseAdapter{
    public void mouseClicked(MouseEvent e) {
      int oldColumn=columnSorted;
      int columnIndexPressed = getColumnModel().getColumnIndexAtX(e.getX());
      if (oldColumn == columnIndexPressed)
        ascendedSorted = !ascendedSorted;
      sortByColumnNr(columnIndexPressed, ascendedSorted);
    }
  }
  
}