package graphic.services;

import javax.swing.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import graphic.services.tree.*;

/**
 * This is JQTables Data core.
 */

public class JQTableData extends AbstractTableModel implements Serializable {

  private int size[];
  private boolean visible[];
  private LinkedList contens = new LinkedList();
  private String[] names;

  /**
   * Set-constructor
   *
   * @param   names  ColumnNames
   */
  public JQTableData(String[] names){
    this.names = names;
    size = new int[names.length];
    visible = new boolean[names.length];
    for (int i = 0 ; i <visible.length ; i++)
      visible[i] = true;
  }
  
  /**
   * Sets the size of all columns in the table.
   *
   * @param   i  int[0] is the size of column nr. 0.. and so on.
   */
  public void setSize(int i[]){
    size = i;
  }
  
  protected int[] getSize(){
    return(size);
  }
  
  protected void setVisibleColumn(int column, boolean visible){
    this.visible[column] = visible;
  }
  
  /**
   * The method only hides the data, it does NOT delete the data from the model.
   *
   * @param   column  columnnr
   * @return     true/false
   */
  public boolean isColumnVisible(int column){
    return visible[column];
  }

  /**
   * The size of a column
   *
   * @param   i  columnr
   * @return     the width
   */
  public int getSize(int i){
    return size[i];
  }
  

  /**
   * Gets columnname
   *
   * @param   col  columnnr
   * @return     The name of the column
   */
  public String getColumnName( int col ) {
    return names[col];
  }
  

  /**
   * Replaces a row in the table. Starts at 0.
   *
   * @param   place  The post to replace
   * @param   obj  The new post.
   * @return     Wether it went well
   */
  public boolean replace(int place, Object[] obj){
    if (obj.length == names.length ){
      contens.set(place, obj);
      fireTableDataChanged();
      return true;
    }
    else
      return false;
  }  
  

  /**
   * Adds a post to the table at a certain position 0=in front
   *
   * @param   place  the position
   * @param   obj  the post to insert
   * @return     Wether it went well.
   */
  public boolean add(int place, Object[] obj){
    if (obj.length == names.length ){
      contens.add(place, obj);
      fireTableDataChanged();
      return true;
    }
    else
      return false;
  }  


  /**
   * Adds a post to the table
   *
   * @param   obj  the post to insert
   * @return     Wether it went well
   */
  public boolean add(Object[] obj){
    if (obj.length == names.length ){
      contens.add(obj);
      fireTableDataChanged();
      return true;
    }
    else
      return false;
  }
  

  /**
   * The number of columns.
   *
   * @return     column count.
   */
  public int getColumnCount() {
    return names.length;
  }
  

  /**
   * Number of visible columns
   *
   * @return     visible column count
   */
  public int getVisibleColumnCount(){
    int visibleColumns = 0;
    for (int i = 0 ; i < visible.length ; i ++)
      if (visible[i])
        visibleColumns++;
    return visibleColumns;
  }

  /**
   * Number of rows.
   *
   * @return     row count
   */
  public int getRowCount() {
    return contens.size();
  }

  public Object getValueAt( int row, int col ){
    Object[] temp = (Object[]) contens.get(row);
    return temp[col];
  }


  /**
   * Gets a row in the table
   *
   * @param   row  posten der nskes hentet
   * @return     posten der blev hentet
   */
  public Object getRow(int row){
    return contens.get(row);
  }
  

  /**
   * Resets the contens of the table. All data will be lost!
   *
   */
  public void reset(){
    contens.clear();
    fireTableDataChanged();
  }
  

  /**
   * Removes a row in the table
   *
   * @param   i  row number
   */
  public void remove(int i){
    contens.remove(i);
    fireTableDataChanged();
  }
  

  /**
   * Updates the table graphically.
   *
   */
  public void update(){
    fireTableDataChanged();
  }
	
  /**
   * Sorts all the rows in the dataset after a column ascending or descending.
   *
   * @param   columnNr  column number
   * @param   ascending  true=ascending, false=descending.
	 * @param   ts  How to sort the columns
   */
  protected void sortByColumnNr(int columnNr, boolean ascending, JQTableSortMethod ts){
    
    for (int i = 0 ; i < columnNr ; i ++)
      if(!visible[i])
        columnNr++;    
    
    //Makes a tree that can sorts the posts
    BinTree bt = new BinTree();
    for (int i = 0 ; i < contens.size() ; i ++){
      JQTableTreeNode jqttn = new JQTableTreeNode( (Object[]) contens.get(i), ts );
      jqttn.setSortingColumn(columnNr);
      bt.insert(jqttn);
    }
    
    if (ascending)//ascending
      contens = bt.udskriv();
    else{//descending
      contens.clear();
      LinkedList ll= bt.udskriv();
      for (int i = ll.size() ; i > 0 ; i--)
        contens.add(ll.removeLast());
    }
    update();
  }
	
	class JQTableTreeNode implements TreeNode{
	  
	  private Object[] element;
	  private TreeNode left;
	  private TreeNode right;
	  private int columnNr;
	  private JQTableSortMethod ts;

    public JQTableTreeNode(Object[] element, JQTableSortMethod ts){
	   this.element=element;
	   this.ts = ts;
	  }
	  public void setSortingColumn(int columnNr){
	    this.columnNr = columnNr;
	  }
	  
	  public String getSortingQualifier(){
	    return element[columnNr].toString();
	  }
	  
	  public void setLeftNode(TreeNode left){
	    this.left = left;
	  }

	  public void setRightNode(TreeNode right){
	    this.right = right;
	  }

	  public TreeNode getLeftNode(){
	    return left;
	  }

	  public TreeNode getRightNode(){
	    return right;
	  }
	  
	  public Object getElement(){
	    return element;
	  }
	  
	  public int compareTo(TreeNode treeNode){
	    
	    if (ts!= null){
	      int comp = ts.compareTo(getSortingQualifier(), treeNode.getSortingQualifier());
	      if (comp != -2)
	        return comp;
	    }
	    
	    try{
	      int num1 = Integer.parseInt(getSortingQualifier());
	      int num2 = Integer.parseInt(treeNode.getSortingQualifier());
	      
	      if (num1 == num2)
	        return 0;
	      if (num1 < num2)
	        return -1;
	      else
	        return 1;
	      
	    }catch(NumberFormatException nfe){
	      return this.getSortingQualifier().toLowerCase().compareTo(treeNode.getSortingQualifier().toLowerCase());
	    }
	  }
	}
}