package server;

import java.io.*;
import java.net.*;
import java.util.*;

import setup.*;
import server.event.*;

/**
 * The configuration file setup.conf is read though this class. The class contains
 * a number of servicemethods to fetch different values through the configurationfile.
 */

public class Conf{
  
  private static XMLParser parser;
  private static XMLTag xml;
  private static Conf conf;
	private static BigBrother bb = BigBrother.getInstance();
	private static HashSet observers = new HashSet();
  

	/**
	 * Singleton class
	 *
	 */
  private Conf(){
		reload();
  }
	
	public static void addObserver(ConfObserver co) {
		observers.add(co);
	}

	public static boolean removeObserver(ConfObserver co) {
		return observers.remove(co);
	}

	/**
	 * Returns the entire "root" of the server. Amongst other things, the tag contains the Server-Tag.
	 *
	 * @return     root.
	 */
	public static XMLTag getRoot() {
		return xml;
	}
	

	/**
	 * How often the server should check the clients for beeing passive
	 *
	 * @return     00:05:00 if no tag is found (the tag is also added i this case)
	 */
	public static String getCheckIniterval() {
		try {
			return xml.getQuickOptionValue("Intelligence.checkIniterval");
		}catch(NoSuchOptionException e) {
			try {
				xml.getTagLike("Intelligence").addOption("checkIniterval", "00:05:00");
			}catch(NoSuchTagException e1) {
				//This exception is theoretical, and cannot occur.
			}
		}catch(NoSuchTagException e) {
			XMLTag tag = new XMLTag("Intelligence");
			xml.addTag(tag);
			tag.addOption("checkIniterval", "00:05:00");
		}
		return "00:05:00";
	}
	

	/**
	 * Sets the how often the server should check the clients for beeing passive
	 *
	 * @param   time  eg. 00:05:00
	 */
	public static void setCheckIniterval(String time) {
		try {
			xml.setQuickOptionValue("Intelligence.checkIniterval",time);
		}catch(NoSuchTagException e) {
			getCheckIniterval();//Makes the required structure
			setCheckIniterval(time);//Tries again
		}		
	}
	

	/**
	 * Is server intelligence for kicking and banning clients turned on?
	 *
	 * @return     false if no tag is found (the tag is also added i this case)
	 */
	public static boolean IsIntelligenceActive() {
		try {
			return xml.getQuickOptionValue("Intelligence.activated").equals("yes");
		}catch(NoSuchOptionException e) {
			try {
				xml.getTagLike("Intelligence").addOption("activated", "no");
			}catch(NoSuchTagException e1) {
				//This exception is theoretical, and cannot occur.
			}
		}catch(NoSuchTagException e) {
			XMLTag tag = new XMLTag("Intelligence");
			xml.addTag(tag);
			tag.addOption("activated", "no");
		}
		return false;
	}
	

	/**
	 * Sets server intelligence for kicking and banning active
	 *
	 * @param   active  on/off
	 */
	public static void setIntelligenceActive(boolean active) {
	
		String val = "no";
		if (active)
			val = "yes";
		
		try {
			xml.setQuickOptionValue("Intelligence.activated",val);
		}catch(NoSuchTagException e) {
			IsIntelligenceActive();//Makes the required structure
			setIntelligenceActive(active);//Tries again
		}
	}
	
	public static int getCommandUsage(String name) throws Exception{
			return Integer.parseInt(xml.getQuickOptionValue("Memory.CommandUsage."+name));
	}
	
	public static void setCommandUsage(String name, int val) {
		String value =""+val;
		XMLTag memTag,comTag;
		try {
			memTag = xml.getTagLike("Memory");
		}catch(NoSuchTagException nste) {
			memTag = new XMLTag("Memory");
			xml.addTag(memTag);
		}
		
		try {
			comTag = xml.getTagLike("Memory");
		}catch(NoSuchTagException nste) {
			comTag = new XMLTag("CommandUsage");
			xml.addTag(memTag);
		}
		
		try {
			comTag.setOptionValue(name, value);
		}catch(NoSuchOptionException nste) {
			comTag.addOption(name,value);
		}
	}
	
	/**
	 * Gets all active commands. These commands are the ones the client must use if he does not
	 * want to be kicked when serverintelligence is turned on.
	 *
	 * @return     String[] {List, Retr, ect...}
	 */
	public static String[] getActiveCommands() {

		try {
			XMLTag server = xml.getTagLike("Intelligence");
			String all = server.getOptionValue("active_commands");
			StringTokenizer st = new StringTokenizer(all,",");
			String[] commands = new String[st.countTokens()];
			int i = 0;
			while (st.hasMoreTokens()) {
				commands[i++] = st.nextToken().trim();
			}
			return commands;
			
		}catch(NoSuchOptionException e) {
			bb.tagException(e, "Could not find 'Actice Commands'");
		}catch(NoSuchTagException e) {
			bb.tagException(e, "NO Intelligence-Tag deffined. Could not find 'Actice Commands'");
		}
		
		return new String[0];
	}
	

	/**
	 * Sets the active commands. These commands are the ones the client must use if he does not
	 * want to be kicked when serverintelligence is turned on.
	 *
	 * @param   commands  String[] {List, Retr, ect...}
	 */
	public static void setActiveCommands(String[] commands) {
		StringBuffer buf = new StringBuffer(commands.length * 5);
		for (int i = 0 ; i<commands.length-1 ; i++)
			buf.append(commands[i] + ",");
		if (commands.length>0)
			buf.append(commands[commands.length-1]);	
		
		try {
			xml.setQuickOptionValue("Intelligence.active_commands", buf.toString());
		}catch(NoSuchTagException e) {
			try {
				XMLTag tag = xml.getTagLike("Intelligence");
				tag.addOption("active_commands","");
				setActiveCommands(commands);
			}catch(NoSuchTagException nste) {
				xml.addTag(new XMLTag("Intelligence"));
				setActiveCommands(commands);
			}
		}
		
	}
 
	/**
	 * Searches for users i in both 'Users.User' and 'Users.Usergroup'.
	 *
	 * @return     all the servers users
	 */
  public static XMLTag[] getUsers(){
    XMLTag[] arr = new XMLTag[0];
		try{
      arr = xml.getTagsLike("Users.User");
    }catch(NoSuchTagException e){
			//There were no users, that were not in a usergroup
    }
		
		XMLTag[] groups = getGroups();
		
		for(int i = 0 ; i < groups.length ; i++) {
			try {				
					arr = concat(arr,groups[i].getTagsLike("User"));
	    }catch(NoSuchTagException e){
	      //There were no users in the Usergroup
	    }
		}

		return arr;
  }
	

	/**
	 * Looks for usergroups in 'Users.Usergroup'
	 *
	 * @return     all usergroups on the server.
	 */
	public static XMLTag[] getGroups() {
		try {
			return xml.getTagsLike("Users.Usergroup");
		}catch(NoSuchTagException nste) {
			return new XMLTag[0];
		}
	}
	

	/**
	 * Concatinates to arrays of XMLTags. oldArr will be first in the new array,
	 * and newArr last.
	 *
	 * @param   newArr  The new array, that should be added to the old array
	 * @param   oldArr  the old array
	 * @return     The to arrays concatinated to a single array.
	 */
	public static XMLTag[] concat(XMLTag[] newArr, XMLTag[] oldArr) {
		XMLTag[] arr = new XMLTag[oldArr.length+newArr.length];
		int count = 0;
		for(;count < oldArr.length ; count++)
			arr[count]=oldArr[count];
		for(int i = 0 ; i < newArr.length ; i++)
			arr[count+i]=newArr[i];	
		
		return arr;	
	}
  

	/**
	 * Adds a new user to the 'Users' tag.
	 *
	 * @param   name  The users name (for reference use)
	 * @param   login  loginnavn
	 * @param   pass  password
	 */
  public static void addUser(String name, String login, String pass){
    try{
      XMLTag ny = new XMLTag("User");
      ny.addOption("name",name);
      ny.addOption("username",login);
      ny.addOption("password",pass);

      xml.getTagLike("Users").addTag(ny);
    }catch(NoSuchTagException e){
	    bb.tagException(e, "Could not add the new user: [" + name + "]");
    }
  }
  

	/**
	 * The port the server is configured to start up on.
	 *
	 * @return     Configured port, or 21 if not port is found.
	 */
  public static int getServerPort(){
    try{
      return Integer.parseInt(xml.getQuickOptionValue("Server.port"));
    }catch(Exception e){
      bb.tagException(e, "Using standard port 21");
    }
    return 21;
  }
	

	/**
	 * Gets the sessionnumber on the server.
	 * If the session number isn't found, 0 are returned, and the tag 'Memory' with 
	 * option 'sessions' 0 are added to the setup.conf file
	 *
	 * @return     sessionnumber or 0.
	 */
	public static int getSession(){
    try{
      return Integer.parseInt(xml.getQuickOptionValue("Memory.sessions"));
    }catch(NoSuchOptionException o) {
			try {
				xml.getTagLike("Memory").addOption("sessions", "0");
			}catch(NoSuchTagException nst) {}
		}catch(NoSuchTagException t){
			XMLTag tag = new XMLTag("Memory");
			tag.addOption("sessions", "0");
      xml.addTag(tag);
    }
    return 0;
  }
	

	/**
	 * Sets the value in the 'Memory.sessions'
	 *
	 * @param   seNr  sessionnumber.
	 */
	public static void setSession(int seNr) {
		try{
			xml.setQuickOptionValue("Memory.sessions", ""+seNr);
		}catch(Exception e){
      bb.tagException(e, "The option 'sessions' could not be updated with ["+seNr+"] i the file Setup.xml");
    }
	}
	

	/**
	 * The ip, that is informed the client by a 'pasv' command.
	 *
	 * @return     ip / null if not found
	 */
	public static String getServerIP(){
    try{
      return xml.getQuickOptionValue("Server.ip");
    }catch(NoSuchTagException e){
      return null;
    }
	}

	/**
	 * The initerval of ports, that may be used by passiv filetransfer. If nothing
	 * is found the initerval 1025-65536 is returned (both ports included)
	 *
	 * @return     {startport,endport}
	 */
	public static int[] getPassivPorts(){
		int[] arr =  {1025,65536};
    try{
      String val = xml.getQuickOptionValue("Server.passivports");
			arr[0] = Integer.parseInt(val.substring(0,val.indexOf('-')));
			arr[1] = Integer.parseInt(val.substring(val.indexOf('-')+1, val.length()));
			return arr;
			
    }catch(NoSuchTagException e){
      return arr;
    }catch(NumberFormatException e) {
			bb.tagException(e, "Passiv PortRange is: " + arr[0] + " to " + arr[1] + ".");
			return arr;
    }
	}
	

	/**
	 * The maximum number of uses on the server. 
	 *
	 * @return     10.000 if nothing is found.
	 */
	public static int getMaxUsers(){
    try{
      return Integer.parseInt(xml.getQuickOptionValue("Server.max_total_users"));
    }catch(NoSuchTagException e){
			bb.tagException(e, "Max number of users set to 10000");
      return 10000;
		}catch(NumberFormatException e){
			bb.tagException(e, "Max number of users set to 10000");
      return 10000;
    }
	}

	/**
	 * Seeks out an users option. If it isn't found, the search goes on in the
	 * 'Usergroup', and then in the 'Server' tag
	 *
	 * @param   t  The users tag
	 * @param   option  The option that is seeked after
	 * @return     An empty string ("") if nothing else is found.
	 */
	public static String getUserOption(XMLTag t,String option){
		try {
			return t.getOptionValue(option);
		}catch(NoSuchOptionException nsoe1) {
			XMLTag parent = t.getParent();
			try {
				if (parent != null && parent.tagName.equals("Usergroup"))
					return t.getParent().getOptionValue(option);
			}catch(NoSuchOptionException nsoe2) {}
			
			try {
				return xml.getQuickOptionValue("Server."+option);
			}catch(NoSuchOptionException nsoe3) {
//				bb.tagException(nsoe3, "conf:"); //Thrown when a new user is created
			}catch(NoSuchTagException nste3) {
				bb.tagException(nste3, "conf:");
			}
		}
		return "";
	}
	

	/**
	 * All directories tags for a user and its usergroups. 
	 * The users own directories are first in the returned array!
	 *
	 * @param   user  the users tag
	 * @return     if nothing is found, XMLTag[0] are returned...
	 */
	public static XMLTag[] getDirectoriesFromUser(XMLTag user){
		XMLTag[] usr = new XMLTag[0];
		try {
		  usr = user.getTagsLike("Directory");
		}catch(NoSuchTagException nste) {
		}
		try {
			XMLTag parent = user.getParent();
			if (parent.tagName.equals("Usergroup")) {
				XMLTag[] grp = parent.getTagsLike("Directory");
				usr = concat(grp,usr);
			}
		}catch(NoSuchTagException nste) {	
		}
		return usr;
	}
	

	/**
	 * Finds out if the IP, that this socket is connected to is banned.
	 *
	 * @param   client  Socket to client
	 * @return     true, if banned!
	 */
	public static boolean isBanned(Socket client) {
		return isBanned(client.getInetAddress().getHostAddress());
	}
	
	
	/**
	 * Finds out if the IP address, that is passed is banned from this server.
	 *
	 * @param   ipAddress  fx. 127.0.0.1
	 * @return     true, if banned!
	 */
	
	public static boolean isBanned(String ipAddress) {
		String ban="";
		try{
		  ban = xml.getQuickOptionValue("Server.banned");
		}catch(NoSuchTagException e){
		  return false;
		}
		StringTokenizer str = new StringTokenizer(ban, ",");
		while (str.hasMoreTokens()) {
			if (str.nextToken().trim().equals(ipAddress.trim())) {
				return true;
			}
		}
		return false;				
	}
	

	/**
	 * Banns an IP-address, by writing it into the configurationfile.
	 *
	 * @param   address  The IP-address!
	 */
	public static void banAddress(InetAddress address) {
		String ban="";
		try{
      ban = xml.getQuickOptionValue("Server.banned");
			if (ban.indexOf(address.getHostAddress()) == -1) {
				if (!ban.equals(""))
					ban+=", ";
				ban+=address.getHostAddress();
				xml.setQuickOptionValue("Server.banned", ban);
			}		
			}catch(NoSuchTagException e){
		}		
		
	}
	

	/**
	 * The maxupload of the server. Used for grahical purposes only!
	 * 
	 *
	 * @return     liniekapacity or 100.000 KB if not found
	 */
	public static int getLineCapacity() {
		try{
      return Integer.parseInt(xml.getQuickOptionValue("Server.linecapacity"));
    }catch(NoSuchTagException e){
      bb.tagException(e, "conf: Did not find any linekacacity. Uses 100.000 KB.");
			return 100000;
    }
		
	}
  

	/**
	 * Saves the configurationfiles current state onto the harddrive.
	 *
	 */
  public static void save(){
    try{
      parser.save();
    }catch(IOException e){
      bb.tagException(e, "conf: Could not save changes in the configuration-filen. All changes has been lost...");
    }
  }
	

	/**
	 * Discards the current state of the configurationfile [setup.xml], and reads the old configuration
	 * into the servers memory.
	 *
	 * All changes that have not been saved are lost!
	 */
	public static void reload() {
		File f = new File("setup.conf");
    try{
		  parser = new XMLParser(f);
      xml = parser.XML;
		}catch(IOException io1){
			bb.tagException(io1, "Could not read the configuration-file [" + f.getAbsolutePath() + "]. The system will attempt to create a default configurationfile");
	    try {
	    	writeFile(f);
				bb.message("The file [" + f.getAbsolutePath() + "] were sucessfully created");		
	    }catch(IOException io2) {
				bb.tagException(io2, "Could not create the configuration-file [" + f.getAbsolutePath() + "]");
	    }catch(NoSuchTagException te) {
				bb.tagException(te, "An error in the configuration-file [" + f.getAbsolutePath() + "] were detected!");
	    }
    }catch(NoSuchTagException te){
	    bb.tagException(te, "An error in the configuration-file [" + f.getAbsolutePath() + "] were detected!");
    }
		
		Iterator i = observers.iterator(); 
		while (i.hasNext()) {
			((ConfObserver)i.next()).configurationFileReloaded();
		}
		
	}
  

	/**
	 * Returns a singleton of this class. Actually this isn't necessary, as all methods in the class
	 * are declared static.
	 *
	 * @return     Conf instans.
	 */
  public static Conf getInstance(){
    if (conf == null)
      conf = new Conf();
    return conf;
  }
	

	/**
	 * Writes a default configuration-file to the harddrive. This happens
	 * startes uden at en konfigurationsfil eksisterer i forvejen!
	 *
	 * @param   f  the file, that the configuration-file is written to.
	 * @exception   IOException  thrown if the file cannot be created.
	 * @exception   NoSuchTagException  Theorectical, because the XMLParser can throw this exception
	 * parsing the new file. This will NOT happen on a new file.
	 * da filen er tom.
	 */
	private static void writeFile(File f) throws IOException, NoSuchTagException{
		f = new File("setup.conf");
		f.createNewFile();
		parser = new XMLParser(f);				
		xml = parser.XML;
		
		XMLTag server = new XMLTag("Server");
		server.addOption("port","21");
		server.addOption("ip","");
		server.addOption("allow_bounce_attack","no");
		server.addOption("log","yes");
		server.addOption("passivports","1026-65536");  
    	server.addOption("linecapacity", "32");
		server.addOption("max_client_upload","-1");
		server.addOption("max_client_download","-1");
		server.addOption("max_logins_per_user","1");
		server.addOption("max_total_users","100");
		server.addOption("banned","");
		server.addComment("All options marked with * can be assigned at userlevel.");
		server.addComment("");
		server.addComment("[port] states the port the server will run on. Uses 21 as default");
		server.addComment("[ip] The ip, that is informed the client by a 'pasv' command. If nothing is applied, the server uses its own IP (fx. 127.0.0.1)");
		server.addComment("[allow_bounce_attack]* states if the server will make a connection to the ip, that is stated during the 'port' command.(yes/no)");
		server.addComment("[log]* Should all activity made by client be logged as default? (yes/no)");
		server.addComment("[passivporte] The initerval of ports, that may be used by passiv filetransfer. Both ports are incl. (from-to)");
		server.addComment("[linecapacity] The gratest capacity that the server can upload/download. The value are used for graphical purposes. Stated in KB.");
		server.addComment("[max_client_upload]* and [max_client_download]* is unlimited at a value of 0 or less. (stated in KB. fx. 10)");
		server.addComment("[max_logins_per_user]* The number of logins one user may make from one IP.");
		server.addComment("[max_total_users] is set to 10000 if nothing else is applied.");
		server.addComment("[banned] states comma-seperated IP-addresses, that is denied access to the server! (fx. 127.0.0.2, 192.168.1.4)");
		xml.addTag(server);
		
		XMLTag huk = new XMLTag("Memory");
		huk.addOption("sessions", "0");
		huk.addComment("");
		huk.addComment("Setups that the server will remember between runs...");
		xml.addTag(huk);

		XMLTag intelligence = new XMLTag("Intelligence");
		intelligence.addOption("activated", "no");
		intelligence.addOption("active_commands", "Appe,List,Nlst,Retr,Stor");
		intelligence.addOption("checkIniterval", "00:05:00");
		intelligence.addComment("");
		intelligence.addComment("The servers ban/kick intelligence...");
		xml.addTag(intelligence);
		
		XMLTag brugere = new XMLTag("Users");
		brugere.addComment("");
		brugere.addComment("Users on the server.");
		xml.addTag(brugere);
		
		XMLTag bruger = new XMLTag("User");
		bruger.addOption("name", "Anonymous Users");
		bruger.addOption("username", "Anonymous");
		bruger.addOption("password", "");
		bruger.addOption("log", "no");
		bruger.addComment("");
		bruger.addComment("[name] The name of the user. Only used for convenience of the administator of the server");
		bruger.addComment("[username] states the login that the user must use to log onto the server");
		bruger.addComment("[password] states the password that the user must use to log onto the server");
		bruger.addComment("[log] overrides the other logsetting for the server and turns logging of for this user.");
		brugere.addTag(bruger);
		
		XMLTag bibliotek = new XMLTag("Directory");
		bibliotek.addOption("name","c:"+System.getProperty("file.separator")+"pub");
		bibliotek.addOption("list","yes");
		bibliotek.addOption("read","yes");
		bibliotek.addOption("write","no");
		bibliotek.addComment("");
		bibliotek.addComment("[name] an absolute path. The first path for a user is automatically default by login.");
		bibliotek.addComment("[list] states if the user is allowed to recive a list of the directory (yes/no)");
		bibliotek.addComment("[read] states if the user is allowed to read files in the directory (yes/no)");
		bibliotek.addComment("[write] states if the user is allowed to delete/make/change files and directories in the directory (yes/no)");
		bruger.addTag(bibliotek);
		parser.save();
	}
}