package server;

import java.io.*;
import java.util.*;
import server.event.*;
import server.services.*;

/**
 * An object, that are capeable of reading a logfile from an user. The Object can <b>ONLY</b>
 * read a logfile from <b>one</b> user.
 */

public class LogFileReader extends Observable {
	
	private File file;
	private String login;
	private BufferedReader bf;
	private static HashMap logFiles = new HashMap();

	/**
	 * Set Constructor that makes a LogFileReader, which is capeable of reading a
	 * logfile from a user with the passed login
	 *
	 * @param   login  loginname.
	 */
	public LogFileReader(String login) {
		this(new File("log"+System.getProperty("file.separator")+login+".log"),login);
	}
	
	/**
	 * This constructor is made primarily to read the ban-patterns.conf file. Although it can read all logfiles handed to it.
	 * In this case, the login is not used for anything else than a nice identification for the programmer...
	 *
	 * @param   file  file to read.
	 * @param   login  the loginname, that this logfile represents.
	 */
	public LogFileReader(File file, String login) {
		this.login = login;
		this.file = file;
		
		HashSet h = (HashSet) logFiles.get(file);
		if (h==null)
			h = new HashSet();
		h.add(this);	
		
		logFiles.put(file,h);
	}
	

	/**
	 * Checks wether this LogFileReader contains the passed LogFileSession.
	 * The sessions are compared with the LogFileSession.equals(LogFileSession) method.
	 *
	 * @param   ses  Session to find
	 * @return     true if found
	 */
	public boolean contains(LogFileSession ses) {
		int num[] = getSessionNumbers();

		for (int i = 0 ; i < num.length ; i++)
			if(getSession(num[i]).equals(ses))
				return true;
		
		return false;	
	}

	/**
	 * The login, som that the object were instantiated with.
	 *
	 * @return     loginname
	 */
	public String getLogin() {
		return login;
	}
	

	/**
	 * Gets a session with the passed number.
	 *
	 * @param   i  the sessionnumber that should be found.
	 * @return     null, if the sessionnumber is not found
	 */
	public LogFileSession getSession(int i) {
		try {
			bf = new BufferedReader(new FileReader(file));
			String l = "";
			l = bf.readLine();
			while (l!=null){
				if(!(l.equals("") || l.startsWith("*"))) {
					String[] o = getLine(l);
					if (o[1].equals(i + "")) {
						String ip;
						String tid;
						Vector v = new Vector(100);
						o = getLine(bf.readLine());
						if (o[0].equals("IP"))
							ip = o[1];
						else
							break;
							//return null;
						
						o = getLine(bf.readLine());
					
						if (o[0].equals("Time"))
							tid = o[1];
						else
							break;
							//return null;
						
						l = bf.readLine();
						if (!l.equals("****************************************************************************************"))
							break;
							//return null;
						
						l = bf.readLine();
						while (l.startsWith("*C") ||l.startsWith("*S")) {
							v.add(l.substring(1,l.length()));
							l = bf.readLine();
						}
						
						if (!l.equals("****************************************************************************************"))
							break;
							//return null;
						bf.close();
						return new LogFileSession(login,i,ip,tid,v);
					}
				}
				l = bf.readLine();
			}
			bf.close();
			return null;
		}catch(IOException e) {
			BigBrother.getInstance().LogFileExcption(e, "Could not read the logfile for the user:" + login); 
			return null;
		}
	}
	

	/**
	 * Deletes LogFileSessions from a user. If the method returns false, and something
	 * in the process has made the logfile inconsistent, a file named xxx.log.bakcup (where xxx=login)
	 * will be made. This file contains the original information.
	 *
	 * @param   logFileSessions  The sessions that should be deleted
	 * @return     true if the deletion went well
	 */
	public boolean deleteSessions(LogFileSession[] logFileSessions) {
		//Builds the vector "all" and removed all LogFileSessions from is, that should be deleted.
		int[] num = getSessionNumbers();
		Vector all = new Vector(num.length);
		for(int i = 0 ; i < num.length ; i++)
			all.add(getSession(num[i]));
		
		LogFileSession element;
		for (int i = all.size()-1 ; i >= 0 ; i-- ) {
			element = (LogFileSession) all.get(i);
			for(int j = 0 ; j < logFileSessions.length ; j++)
				if (element.equals(logFileSessions[j]))
					all.remove(element);		
		}	
		
		//Renames to safetycopy (.backup), and deletes the old logfile
		String name;
		if (file.getParent() == null)
			name = file.getName();
		else
			name = file.getParent()+System.getProperty("file.separator")+file.getName();
		name+=".backup";
		
		File safe = new File(name);
		
		if (!file.renameTo(safe)) {
			System.out.println("Filen [" + file.getAbsolutePath() + "] kunne ikke omdbes til [" + safe.getAbsolutePath() + "]");
			return false;
		}
			
		//writes the new logfile
		int lng = all.size();
		LogFileWriter writer = new LogFileWriter();
		for (int i = 0 ; i < lng ; i++) {
			if(!writer.write((LogFileSession)all.get(i),file))
				return false;
		}
		
		//deletes the safetycopy if all goes well.
		boolean ok = safe.delete();
		if (ok) {
			setLogFileUpdate(null,file);
		}
		
		return ok;
	}
	

	/**
	 * Notifies all obervers on the file (LogFile), that change has been made to the logFile.
	 *
	 * @param   arg  The LogFileSession that is added, or null if a LogFileSession has been deleted
	 * @param   file  the file that has been updated
	 */
	public static void setLogFileUpdate(Object arg, File file) {
		HashSet h = (HashSet) logFiles.get(file);
		if (h!=null) {
			Iterator it = h.iterator();
			LogFileReader o;
			while (it.hasNext()) {
				o = (LogFileReader) it.next();
				o.setChanged();
				o.notifyObservers(arg);
			}
		}
	}

	
	/**
	 * Reads the logfile, and findes numbers of the sessions contained in the file.
	 *
	 * @return     an empty array, if there are no sessions in the file, nulll if the Logfile is not found
	 */
	public int[] getSessionNumbers() {
		Vector v = new Vector();
		try {
			bf = new BufferedReader(new FileReader(file));
			String l = "";
			l = bf.readLine();
			while (l!=null){
				if(!(l.equals("") || l.startsWith("*"))) {
					String[] o = getLine(l);
					if (o[0].equals("SessionNr")) {
						try {		  
							v.add(""+Integer.parseInt(o[1]));
						}catch(NumberFormatException e) {/*ignore this session! The number is not valid!*/}
					}
				}
				l = bf.readLine();
			}
			bf.close();
			int[] s = new int[v.size()];
			for(int i = 0 ; i < s.length ; i++)
				s[i] = Integer.parseInt(v.get(i).toString());
			
			return s;	
		}catch(IOException e) {
			BigBrother.getInstance().LogFileExcption(e, "Kunne ikke lse Logfilen for bruger:" + login); 
			return null;
		}		
	}
	

	/**
	 * Can get a line, that are formatted, as one of the tree lines below:<br>
	 * SessionNr: [4]<br>
	 * IP       : [127.0.0.1]<br>
	 * Time     : [D. 23.05.2002  kl. 18:00:27]<br><br>
	 * By line one {"SessionNr","4"} is returned
	 *
	 * @param   line  input
	 * @return     array...
	 */
	private String[] getLine(String line) {
		String[] s = new String[2];
		s[0] = line.substring(0,line.indexOf(':')).trim();
		s[1] = line.substring(line.indexOf('[')+1, line.indexOf(']'));
		return s;
	}
}