package server;

public interface SurvailanceController {

	/**
	 * Should validate the clients commands and return true if the command 
	 * lets the client continue his session. 
	 * If this method returns true one time within the 'Survailance.secondsToTimeout', 
	 * the client will NOT be kicked off the server. If the the method 
	 * does not return true one or more times within the 'Survailance.secondsToTimeout', 
	 * the client is automatically kicked!
	 *
	 * @param   command  the command the client sends to the server
	 * @param   userSession  the client that sended the command
	 * @return     true if the client may continue his session.
	 */
	public boolean letSessionContinue(String command, UserSession userSession);


	/**
	 * Should find out if the command results in a disconnection of the client.
	 * If true is returned, the client is disconnected at once.
	 *
	 * @param   command  the command the client sends to the server
	 * @param   userSession  the client that sended the command
	 * @return     true if the client should be disconnected.
	 */
	public boolean disconnectSession(String command, UserSession userSession);
	
}