package server.event;

import java.io.File;
import server.UserSession;
/**
 * Must be implemented by those, who wishes to listen on a UserSession-Object (a client).
 */

public interface UserSessionObserver {

public static final int USER_ONLINE = -1;
public static final int USER_NORMAL_END = 0;
public static final int USER_KICKED = 1;
public static final int USER_TIMEOUT = 2;
public static final int USER_BANNED = 3;
public static final int USER_AUTO_BANNED = 4;
public static final int USER_KICKED_DUE_TO_SERVER_STOP = 5;


	/**
	 * All the client sends to the server is send though here as an echo!
	 *
	 * @param   command  command recived from the client
	 */
	public void commandUpdate(String command);
	
	/**
	 * All the server sends to the client is send though here as an echo!
	 *
	 * @param   response  response send to the client
	 */
	public void serverResponse(String response);

	/**
	 * When a fileTransfer has started, the method is called.
	 *
	 * @param   filename  The file, that is beeing uploaded or downloaded.
	 */
	public void fileTransferStarted(File filename);
	

	/**
	 * When a fileTransfer has ended, the method is called.
	 *
	 * @param   successfull  true=success, false=abort
	 */
	public void fileTransferEnded(boolean successfull);
	

	/**
	 * When a file is beeing transfered, this method is called every time the server
	 * has transfered the number of bytes, that the clients may have per sek.
	 *
	 * @param   data  long[] arr =  {startAt, transfered, file.length()};
	 */
	public void bytesTransmitted(long[] data);
	

	/**
	 * When the user logs in, this method is called.
	 *
	 * @param   usersession  The user that loged in..
	 */
	public void userLogedIn(UserSession usersession);
	

	/**
	 * When a user loges out, this method is called.
	 *
	 * @param   usersession  The user that loged out..
	 */
	public void userLogedOut(UserSession usersession);
	

	/**
	 * When a user gets disconnected, this method is called
	 *
	 * @param   userSession  the user, that is disconnected
	 * @param   reasonCode  the reason for the disconnection. (see the USER_?? statics above)
	 */
	public void userDisconnected(UserSession userSession, int reasonCode);
}