package server.services;

import java.net.*;
import java.io.*;
import java.util.*;

import setup.*;
import server.*;
import server.event.*;

public class Command{
  
	private BigBrother bb;
  private Socket socket, dataSocket;
  private String answer[];
  private boolean dataSocketInUse = false;
	private boolean successfull=true;
  private FileHandler fileHandler;
  private long startAt = 0;
  private Session ses;
  private int sended, recived, sendedBytes, recivedBytes, dataPort=0;
  private PrintWriter out;
  private boolean ClientConnected=false;

  protected Command(Socket socket, Session ses){
    bb = BigBrother.getInstance();
		this.socket = socket;
    this.ses = ses;
    try {
      out = new PrintWriter( new BufferedWriter( new OutputStreamWriter( socket.getOutputStream() ) ),true);
    }
    catch (IOException e){
      bb.commandException(e,"Command: Kan ikke skrive til Klient.. Socket i brug.");
    }

  }
  
  public int[] getFileTrafficInfo(){
    int[] info = {sended,recived,sendedBytes,recivedBytes};
    return info;
  }
    
  public synchronized void run(String answer){
    this.answer = new String[] {answer};
    run();
  }
  
  public synchronized void run(String[] answer){
    this.answer = answer;
    run();
  }
  
  private void run(){
    int i=0;
    try {
      for (i = 0 ; i < answer.length; i++){
        out.println(answer[i]);
        ses.commandUpdate(answer[i],2);
      }
    }
    catch ( NullPointerException e ){
      bb.commandException(e,"Command: Kan ikke skrive til Klient.. Ingen Socket ledig.");
    }
  }
  
  public synchronized void closeSocket(){
    try{
      socket.close();
    }
    catch(IOException ioe){
      bb.commandException(ioe,"Command: DataSocket not closed.");
    }
  }
  
  private boolean getFile(File dest, boolean append){
		if (dataSocketInUse == false && makeDataSocket() ){
    	int max = getMax("max_client_upload");  
		  fileHandler = new FileHandler(dataSocket, startAt, max, this);
			fileHandler.setFileReciver(dest);
    	fileHandler.append(append);
			ses.fileTransferStarted(fileHandler);
    	fileHandler.start();
      return true;
    }
    else
      return false;
  }
  
  public boolean sendFile(File dest){
		if (dataSocketInUse == false && makeDataSocket() ){
      int max = getMax("max_client_download");
			fileHandler = new FileHandler(dataSocket, startAt, max, this);
			fileHandler.setFileSender(dest);
			ses.fileTransferStarted(fileHandler);
      fileHandler.start();
      return true;
    }
    else
      return false;
  }
	
	public boolean sendList(byte[] b) {
		if (dataSocketInUse == false && makeDataSocket() ){
      int max = getMax("max_client_upload");
			fileHandler = new FileHandler(dataSocket, startAt, max, this);
			fileHandler.setListSender(b);
      fileHandler.start();
      return true;
    }
    else
      return false;
	}
  
  public boolean getFile(File dest){
  	return getFile(dest, false);
  }
  
  public boolean appendToFile(File dest){
  	return getFile(dest, true);
  }

	private int getMax(String max) {
		String res = Conf.getUserOption(ses.xml, max);
		try {
			return Integer.parseInt(res);
		}catch(NumberFormatException nf) {
			bb.commandException(nf,"There was an error in the File setup.conf. A number was expected by "+ max +" tagget. Found instead:["+res+"]. Using unlimites speed for the user [" + ses.getLogin() + "].");
			return -1;
		}
	}
	    
  public synchronized void ClientConnected(boolean ClientConnected){
    this.ClientConnected = ClientConnected;
    notifyAll();
  }
  
  private synchronized boolean makeDataSocket(){
		if(ses.mode && ses.dataPort != 0){
	    try{
	      dataSocketInUse=true;
	      dataSocket = new Socket(ses.dataIP, ses.dataPort);
	      return true;
	    }
	    catch(IOException ioe){
	      bb.commandException(ioe,"Command: " + ioe);
	      return false;
	    }
	  }else if(ses.dataPort != 0 && !ses.mode){
        if (!ClientConnected)
          try{
            wait();
          }catch(InterruptedException ie){}
        if (!ClientConnected)
          return false;
        dataSocketInUse=true;
        ClientConnected=false;
	      return true;
	  }else
	  	return false;
  }
  
  public void makeDataSocket(Socket dataSocket){
    this.dataSocket = dataSocket;
  }
  
  public void makePasiveServer(PortCalc pc){
  	if(ses.dataPort != 0)
	  	new PasiveServer(ses.dataPort, pc, this).start();
  }
  
  public boolean DataSocketInUse(){
    return dataSocketInUse;
  }
  
  public void endedDataTransmission(int sendedBytes, int recivedBytes){
    if (sendedBytes > 0){
      this.sendedBytes += sendedBytes;
      this.sended++;
    }
    if (recivedBytes > 0){
      this.recivedBytes += recivedBytes;
      this.recived++;
    }
      
    try{
      dataSocket.close();
      dataSocket=null;
      dataSocketInUse = false;
      startAt=0;
      ses.dataPort=0;
      if (fileHandler.getSendMode() != fileHandler.MODE_SEND_LIST)
				ses.fileTransferEnded(successfull);
      System.gc();
			successfull=true;
    }
    catch(IOException ioe){
      bb.commandException(ioe,"Dataconnection could not be closed.");
    }
  }
  
  public void abortTransmission(){
  	successfull = false;
		if (fileHandler != null)
			fileHandler.sendOk(false);
  }
  
  public void startAt(long startAt){
  	this.startAt = startAt;
  }
	
	public int[] getStatArray() {
		int[] arr =  {recived,sended,recivedBytes,sendedBytes};
		return arr;
	}
  
  public String[] getStat(){
    String[] stat;
    if (dataSocketInUse && fileHandler.getSendMode() != fileHandler.MODE_SEND_LIST){
      stat= new String[3];
      stat[0] = "    Stored " + recived + " files, " + (recivedBytes/1024 ) + " Kbytes";
      stat[1] = "    Retrieved " + sended + " files, " + (sendedBytes/1024 ) + " Kbytes";
      stat[2] = "    " + fileHandler.getStat();
    }
    else{ 
      stat= new String[2];
      stat[0] = "    Stored " + recived + " files, " + (recivedBytes/1024 ) + " Kbytes";
      stat[1] = "    Retrieved " + sended + " files, " + (sendedBytes/1024 ) + " Kbytes";
    }
    return stat;
  }
	
	public void bytesTransmitted(long[] arr) {
		ses.bytesTransmitted(arr);
	}
}