package server.services;

import java.net.*;
import java.io.*;
import java.util.*;

import server.*;
import server.event.*;

public class Dock implements Runnable{
	
  private int sessionNr = Conf.getSession();
  private Thread th;
  private ServerSocket ss;
  private static BigBrother bb = BigBrother.getInstance();
	private int maxConnections = Conf.getMaxUsers();
	private HashSet probes = new HashSet();
	private static HashMap users = new HashMap();
	private HashSet obs;
	private boolean suspended=false;
	private boolean autoSave=false;
	
	public Dock( HashSet observers ){
		obs = observers;
  }
	
  //off as default
  public void setAutoSave(boolean on) {
  	autoSave = on;
  }
  
  public boolean doesAutoSave() {
  	return autoSave;
  }
  
  public void start(){
    suspended=false;
		th = new Thread(this);
    th.start();
  }
  
  public boolean stop(){
		suspended=true;
		try{
      if (ss != null)
				ss.close();
			  ss = null;	
			return true;
    }catch(IOException e){
      bb.dockException(e,"Could not stop the server.");
			return false;
    }
  }
	
	public boolean isRunning() {
		if (ss == null)
			return false;
		return !ss.isClosed();
	}

  public synchronized void run(){
    Socket socket;
    
    try{
      ss = new ServerSocket(Conf.getServerPort(),100);
			obsServerStarted(true);
      while(true){   
        socket = ss.accept();
				if (!Conf.isBanned(socket) && probes.size() < maxConnections) { //hvis IP er banned, disconnectes klienten straks! Ligeledes, hvis der ikke er flere ledige pladser p serveren.
	        Probe p = new Probe(socket, this, sessionNr++);
	        probes.add(p);
					obsClientConnected(p.ses.us);
	        p.start();
					Conf.setSession(sessionNr);
				}else {
					socket.close();
				}
      }
    }catch(SocketException se) {
			if (!suspended)
				obsServerStarted(false); //server could not be started
    }catch(IOException e){
      bb.dockException(e,"");
    }
		if(suspended)
			obsServerStoped();
  }
	
	public static void userLogedOut(Session ses) {
		//Tller klientens antal logins ned med 1.
		String key = ses.getLogin() + ses.clientCommandSocket.getInetAddress().getHostAddress();
		int i = Integer.parseInt(users.get(key).toString());
		users.put(key,""+(--i));	
	}
	
	public void disconnect(Probe p) {
		String login = p.ses.getLogin();
		if (p.ses.logedIn && !login.equals("")) {
			p.ses.logFile.stop();
			userLogedOut(p.ses);
		}
		//Fjerner klienens session fra listen.
		probes.remove(p);
		
		//Opdaterer observers
		obsClientDisconnected(p.ses.us);
		
		if (autoSave)
			Conf.save();
	}
	
	public static boolean canUserLogin(Session ses) {			
		//kig om brugeren er logget ind allerede
		String key = ses.getLogin() + ses.clientCommandSocket.getInetAddress().getHostAddress();
		Object o = users.get(key);
		int now;
		if (o == null) {
			now = 0;
		}else {
			now = Integer.parseInt(o.toString());
		}
		//se hvor mange gange denne bruger m logge ind.
		String mls = Conf.getUserOption(ses.xml,"max_logins_per_user");
		int mli=-1;
		if (!mls.equals("")) {
			try {
				mli=Integer.parseInt(mls);
			}catch(NumberFormatException nfe) {
				ses.bb.commandoException(nfe, "The value ["+mls+"] i max_logins at the user: " + ses.getLogin() + " were not a number. no limits is set.");
			}
		}
		
		if (mli > now || mli < 0) {
			users.put(key, ""+(++now));
			return true;
		}else
			return false;

	}
	
	public HashSet getUsers() {
		return probes;
	}
	
	public synchronized int getServerPort() {
		if (ss==null) {
			return Conf.getServerPort();
		}else {
			return ss.getLocalPort();
		}
	}
	
	private void obsClientConnected(UserSession us) {
		Iterator i = obs.iterator();
		while (i.hasNext()) {
			((ServerObserver)i.next()).clientConnected(us);
		}	
	}
	
	private void obsClientDisconnected(UserSession us) {
		Iterator i = obs.iterator();
		while (i.hasNext()) {
			((ServerObserver)i.next()).clientDisconnected(us);
		}	
	}
	
	private void obsServerStarted(boolean startOk) {
		Iterator i = obs.iterator();
		while (i.hasNext()) {
			((ServerObserver)i.next()).serverStarted(startOk);
		}	
	}
	
	private void obsServerStoped() {
		Iterator i = obs.iterator();
		while (i.hasNext()) {
			((ServerObserver)i.next()).serverStoped();
		}	
	}
}