package server.services;

import java.net.*;
import java.io.*;
import java.util.*;

import server.event.*;

public class FileHandler extends Thread{

  private Socket socket;
  private File file;
  private Command command;
  private BufferedOutputStream output;
  private BufferedInputStream input;
  private boolean sendOK=true;
	private int sendMode;
	public final static int MODE_SEND_FILE  =0;
	public final static int MODE_RECIVE_FILE=1;
	public final static int MODE_SEND_LIST  =2;
  private boolean append = false;
  private long startAt;
  private int transfered;
  private int max;
	private int newSize;
	private byte[] byteArr;
	private byte[] b, t;
		 
  public FileHandler(Socket socket, long startAt, int max, Command command){
		this.max = max*1024;
		this.newSize = this.max;
    this.socket = socket;
    this.command = command;
    this.startAt = startAt;
  }
  
  public void setFileSender(File file){
		this.file = file;
    try{
      output  = new BufferedOutputStream( socket.getOutputStream() );
      input = new BufferedInputStream( new FileInputStream( file ) );
    }
    catch(IOException ioe){
      BigBrother.getInstance().fileHandlerException(ioe,"FileHandler: Bufferes are not set.");
    }
    sendMode=MODE_SEND_FILE;
  }
  
  public void setFileReciver(File file){
		this.file = file;
    try{
      output = new BufferedOutputStream( new FileOutputStream( file.getAbsolutePath(), true ) );
      input = new BufferedInputStream(socket.getInputStream());
    }
    catch(IOException ioe){
      BigBrother.getInstance().fileHandlerException(ioe,"FileHandler: Bufferes are not set." +ioe);
    }
    sendMode=MODE_RECIVE_FILE;
  }
	
	public void setListSender(byte[] byteArr) {
		this.byteArr = byteArr;
		try{
		  output  = new BufferedOutputStream( socket.getOutputStream() );
		  input = new BufferedInputStream( new ByteArrayInputStream(byteArr) );
		}
		catch(IOException ioe){
		  BigBrother.getInstance().fileHandlerException(ioe,"Kunne ikke skrive listen til klientens socket.");
		}
		sendMode=MODE_SEND_LIST;	
	}
  
  public void append(boolean append){
  	this.append = append;
  }
  
  public String getStat(){
    String text="";
    if (transfered > 0){
      if(sendMode == MODE_SEND_FILE)
        text="Sending file " + file.getName() + " (" + transfered + " of " + file.length() + " bytes)";
      else if (sendMode == MODE_RECIVE_FILE)
        text="Reciving file " + file.getName() + " (" + transfered + " of " + file.length() + " bytes)";
      else
				text="Sending directorylist of " + byteArr.length;
    }
    return text;
  }
  
  //Kan afbryde trden. Sker f.eks. ved 'Abor' Kommando.
	public void sendOk(boolean sendOK){
		this.sendOK= sendOK;
	}
	
	//regner forholdet mellem sleep og array ud {sleep i millisek, arraysize i bytes}
	private int[] getSpeed(double speed) {
		max=(int)speed;
		double sleep;
		double size;
		
		if (speed >= 0 && speed <=32768) {
			sleep= 500;
			size = speed/2;
		}else {
			size = 32768;
			sleep = (size/speed)*1000+2;
		}
		return new int[] {(int)sleep,(int)size};
	}
  
  public void run(){

		int[] speed = getSpeed(max);
		
		b = new byte[speed[1]];		

    if (sendMode == MODE_SEND_LIST)
			command.run( "150 Opening ASCII mode data connection for directory list (" + byteArr.length + " bytes)." );
		else
			command.run( "150 Opening ASCII mode data connection for " + file.getAbsolutePath() + " (" + file.length() + " bytes)." );
      
    try{
      int read;
      Date D1 = new Date();
			if(sendMode == MODE_SEND_FILE)
				input.skip(startAt);
      
      if(!append && sendMode == MODE_RECIVE_FILE){
      	output = new BufferedOutputStream( new FileOutputStream( file ) );
      }
      
      while( sendOK ){
				if (!writeBytes(b,speed[0]))
					break;
				if (max != newSize) {
					speed = getSpeed(newSize);
				  b = new byte[speed[1]];
				}
      }
      input.close();
      output.close();
      end();
      if (sendOK)
      	command.run("226 Transfer complete.");
    }catch(IOException ioe){
      end();
    }catch(InterruptedException ie) {
			end();
    }
  }
	
	private boolean writeBytes(byte[] b, int sleeptime) throws IOException, InterruptedException{
	  int read = input.read( b, 0, b.length );
	  if (read==-1) //fil slut
	  	return false;
		output.write(b ,0, read);
		transfered+=read;
		if (sendMode != MODE_SEND_LIST) {
			long[] arr =  {startAt, transfered, file.length() };
			command.bytesTransmitted(arr); 
		}

		if (max >= 0)
			sleep(sleeptime);
		return true;	
	}
	
	private void end(){
    if(sendMode == MODE_SEND_FILE)
      command.endedDataTransmission(transfered, 0);
    else if (sendMode == MODE_RECIVE_FILE)
      command.endedDataTransmission(0, transfered);
		else
		  command.endedDataTransmission(0, 0);
  }
	
	public int getSpeedInBytesPrSek() {
		return max;
	}
	
	public int getSendBytes() {
		return transfered;
	}
	
	public int getSendMode() {
		return sendMode;
	}
	
	public void setSpeedInBytesPrSek(int bytes) {
		newSize = bytes;
	}
	
	public File getFile() {
		return file;
	}
}