package server.services;

import java.io.*;
import setup.*;
import server.*;
import server.event.*;
import server.services.tree.*;

public class FileSystem {
	private XMLTag xml;
	private XMLTag[] bibs;
	private char sep = System.getProperty("file.separator").charAt(0);
	public File curDir;
	public String curStr="";
	private String noSuccessPath;
	public BigBrother bb = BigBrother.getInstance();
	private String root = "[ROOT]";
	private File rnFromFile;
	
	public FileSystem (XMLTag xml){
		this.xml=xml;
		bibs = Conf.getDirectoriesFromUser(xml);
		
		XMLTag home = getHome();
		if (home != null) {
			try {
				curDir = new File(home.getOptionValue("name"));
				
				try {
					curStr = home.getOptionValue("alias");
					if (curStr.trim().equals(""))
						curStr = curDir.getAbsolutePath();
				}catch(NoSuchOptionException e) {
					curStr = curDir.getAbsolutePath();
				}
				if (!curStr.endsWith(""+sep))
					curStr+=sep;
			}catch(NoSuchOptionException e) {
				bb.tagException(e, "FileSystem: ");
			}
		}else {
			curDir = new RootFile(root);
			curStr= root;
		}
	}
	
	public void renameFileFrom(File file) {
		rnFromFile = convert(file);
	}
	
	public boolean isRenameSet() {
		return rnFromFile != null;
	}
	
	public boolean renameFileTo(File file) {
		boolean ok = rnFromFile.renameTo(convert(file));
		rnFromFile = null;
		return ok;
	}
	
	private XMLTag getHome() {		
			for (int i = 0 ; i < bibs.length ; i++)
				try {
					if (bibs[i].getOptionValue("home").equals("yes"))	
						return bibs[i];	
				}catch(NoSuchOptionException e) {
					//bb.tagException(e, "FileSystem");
				}		
		return null;
	}
	
	public String pwd() {
		return curStr;
	}
	
	private XMLTag findClosestMatch(String value, String option) {
		value = value.toLowerCase();
		int depth=0;
		int place=-1;
		String bib;
		for (int i = 0 ; i < bibs.length; i++) {
			try {
				bib = bibs[i].getOptionValue(option).toLowerCase();
				int d = bib.length();
				if (value.startsWith(bib) && d > depth) {
					depth = d;
					place = i;
				}
			}catch(NoSuchOptionException e) {
//				System.out.println("No Tag was found: " + e);
			}
		}
		if (place == -1)
			return null;
		else
			return bibs[place];	
	}
	
	//Converts any directory (existing or alias) to a existing directory.
	private File convert(File f) {
		String total = f.getAbsolutePath();
		try {
			
			XMLTag nameTag = findClosestMatch(total,"name");
			XMLTag aliasTag= findClosestMatch(total,"alias");
			
			// We dont know what it is, so we don't fix it ;o)
			if (nameTag == null && aliasTag == null)
				return f;
			
			if (nameTag != null && aliasTag != null) {
				int aliasDif;
				int nameDif;
				String name = nameTag.getOptionValue("name");
				nameDif = total.length()-name.length();
				String alias = aliasTag.getOptionValue("alias");
//				System.out.println(f.getAbsolutePath() + "alias:"+ alias+ ", name:" + name);
				if (alias != null) {
					aliasDif = total.length()-alias.length();
					if ( aliasDif > nameDif) {
						return f;
					}
				}
				return makeFileFromTag(aliasTag, f, "alias");
			}else if(nameTag != null) {
				return f;
			}else {
				return makeFileFromTag(aliasTag, f, "alias");
			}
		}catch(NoSuchOptionException e) {
			System.out.println("An ERROR occured in FileSystem: " + e);
			return null;
		}
	}
	
	private File makeFileFromTag(XMLTag xml, File f, String option) throws NoSuchOptionException{
		String total = f.getAbsolutePath();	
		String op = xml.getOptionValue(option);
		String dir = total.substring(op.length(),total.length());
		dir = xml.getOptionValue("name") +sep+ dir;
		return new File(dir);
	}
	
	public File cwd(String place) {
		place = place.replace('/',sep);
		
		if (place.length() > root.length() && place.startsWith(root))
			place = sep+place+sep;
		
		if (curStr.equals(root) && place.startsWith("[") && place.endsWith("]")) {
			place = place.substring(1,place.length()-1);
		}
		String str=null;
		File hd=null;
    if(place.equals(root) || place.equals("root") || place.equals(sep+root+sep)) {
			hd = new RootFile(root);
			str=root;
    }else if(place.startsWith(""+sep) ) {
			if (place.length() > root.length() && place.substring(1,root.length()+1).equals(root))
				place = place.substring(root.length()+2,place.length()-1);
			if (place.startsWith("[")) {
				int i = place.indexOf(']');
				place = place.substring(1, i) + place.substring(i+1,place.length());
			}
			if (place.charAt(0) == sep)
				place = place.substring(1);
			if (!place.endsWith(""+sep))
				place+=sep;	
      hd = new File(place);
			str = place;
    }else if(place.equals("..")) {
			
			int first = curStr.indexOf(1,sep);
			int last  = curStr.lastIndexOf(sep)-1;
			int cur   = curStr.lastIndexOf(sep,last);
			
			if (cur > 0 && curStr.length() > 1 && first != last) {
				str = curStr.substring(0,cur) + sep;
				hd = convert(new File(str));
				
				if (!canList(hd)) {
					hd = null;
				}else {
					boolean ok=false;
					String name;
					String alias;
					for (int i = 0 ; i < bibs.length ; i++) {
						try {
							name = bibs[i].getOptionValue("name");
							alias="";
							
							try {
								alias = bibs[i].getOptionValue("alias");
							}catch(NoSuchOptionException nse) {
								//Quite ok.
							}
							
//							System.out.println("hd("+hd.getAbsolutePath()+") str(" + str + ") name(" + name + ") alias(" + alias + ")");
							if ((str.startsWith(alias) && !alias.equals("")) || (alias.equals("") && hd.getAbsolutePath().startsWith(name) && canList(new File(name))) ) {
								ok =true;//FEJL
							}
						}catch(NoSuchOptionException nse) {
							//Quite ok.
						}
					}
					if (!ok)
						hd=null;
				}	
			}			
    }else if( place.length() >= 2 && place.charAt(1) == ':') {
      str = place;
			if (!place.endsWith(""+sep))
				str+=sep;
			hd = new File(str);
	  }else if (curDir != null){
		
			File f = convert(new File(curStr + sep + place)); //convert tilfjet			
      if (f.exists()) {
				hd = f;
				str = curStr + place + sep;
      }
	  }
		if (hd != null)
			hd = convert(hd);		
		if (hd != null && hd.exists()) {
			if (checkDir(hd, 'L')) {
				curStr = str;
				curDir = hd;
			}else
				noSuccessPath = str;
			return hd;
		}else if(place.equals("..") && !checkDir(hd, 'L') ) {
			curStr = root;
			curDir = new File(root);
			return curDir;
		}else {
			noSuccessPath = str;
		}
		return null;
	}
	
	public String getUnsuccessfullPath() {
		return noSuccessPath;
	}
	
	public boolean cdUp() {
		if (cwd("..") != null)
			return true;
		return false;
	}
	
	public boolean canWrite(File f) {
		return checkDir(f,'W');
	}
	
	public boolean canRead(File f) {
		return checkDir(f,'R');
	}
	
	public boolean canList(File f) {
		return checkDir(f,'L');
	}
	
	//'R' = Read, 'W' = Write, 'L' = List
	private boolean checkDir(File f, char c) {
		if (f == null)
			return false;
	
		if (f.getName().equals(root))
			return true;
		
		boolean ok = false;
		
		String ret="";
		
		if (c == 'R')
			ret = "read";
		else if (c == 'W')
		  ret = "write";
		else if (c == 'L')
			ret ="list";
		else
			return false;	
		
		try {
			int depth=0;
				XMLTag dir = findClosestMatch(f.getAbsolutePath(),"name");
				if (dir == null)
					return false;
		    if (dir.getOptionValue(ret).equals("yes")) {
					 	ok=true;
			  }else {
					ok=false;
				}
			return ok;
		}catch(Exception e) {
			System.out.println("FileSystem.checkDir(): ");
			e.printStackTrace();
//			bb.sessionException(e, "ses: The user "+getLogin()+" has not directories deffined");
			return false;
		}		
	}
	
	//Object[] is containing File Objects. If no directory is chosen, method returns null
	//Tager ikke hjde for om brugeren her m lse/skrive/liste!
	public Object[] getList() {
			String[] filelist;
			BinTreeFile BTF = new BinTreeFile();
			if (curDir == null)
				return null;
			if (curDir.getName().equals(root)) {
				filelist = new String[bibs.length];
				String s = "";
				for(int i = 0 ; i< filelist.length ; i++){
		      try {
						s = bibs[i].getOptionValue("alias");
		      }catch(NoSuchOptionException nse) {
		      }
					if (s.equals("")) {
						try {
							s = bibs[i].getOptionValue("name");
						}catch(NoSuchOptionException nse1) {
							//Problem! Neither alias or name was found!
						}
		      }
					if (s != null && canList(convert(new File(s)))) {
						BTF.insert( new RootFile("["+s+"]") );
					}
		    }
				
			}else {
			  filelist = curDir.list();
		    
		    //Alle filer og biblioteker indsttes i et tr
		    for(int i = 0 ; i< filelist.length ; i++){
		      BTF.insert( new File(curDir.getPath() + "/" + filelist[i]) );
		    }
				
				String alias;
				for(int i = 0 ; i< bibs.length ; i++){
					try {
						alias = bibs[i].getOptionValue("alias");
						if (alias.startsWith(curStr) && alias.lastIndexOf(sep)+1 == curStr.length() && canList(convert(new File(curStr))))
							BTF.insert( new RootFile(alias.substring(alias.lastIndexOf(sep)+1,alias.length())) );
					}catch(NoSuchOptionException e) {
						//None available
					}
				}
				
			}
	    //Tret skrives til et nyt array af objects.
	    Object[] obj = BTF.udskriv();
			return obj;
	}
	
	class RootFile extends File {
		String filename;
		public RootFile(String name) {
			super(name);
			this.filename = name;
		}
		
		public boolean isDirectory() {
			return true;
		}
		
		public String getName() {
			return filename;
		}
		
		public boolean exists() {
			return true;
		}
	}
}