package server.services;

import java.io.*;
import java.util.*;
import java.text.*;

import server.event.*;
import server.*;

public class LogFileWriter{

	private Vector com = new Vector();
	private FileWriter logOut;
	private File logFile;
	private static BigBrother bb = BigBrother.getInstance();
	private String username;	
	
	public LogFileWriter() {
	}
	
	public boolean write(LogFileSession logFileSession, File file) {
		
		try {
			FileWriter output = new FileWriter(file,true);
			
			writeHead(logFileSession.getLogonTime(), logFileSession.getClientIP(), logFileSession.getSessionNr(), output);
			
			Vector v = logFileSession.getConversation();
			int size = v.size();
			for(int i = 0 ; i < size ; i++)
				output.write("*" +v.get(i)+"\n");
				
			output.write("****************************************************************************************\n");
			output.write("\n");
			output.close();
			LogFileReader.setLogFileUpdate(logFileSession, file);
		}catch(IOException e) {
		  bb.LogFileExcption(e,"Could not write sessionnumber " + logFileSession.getSessionNr() + " from user " + logFileSession.getLogin() + " into the file: " + file.getAbsolutePath());
			return false;
		}
		
		return true;	
	}	
	
	public boolean write(LogFileSession logFileSession) {
		return write(logFileSession, new File("log"+System.getProperty("file.separator")+logFileSession.getLogin()+".log"));
	}
	
	private void writeHead(String date, String ip, int sessionNr, FileWriter out) throws IOException {
		out.write("SessionNr: ["+sessionNr+"]\n");
		out.write("IP       : ["+ip+"]\n");
		out.write("Time     : ["+date+"]\n");
		out.write("****************************************************************************************\n");
	}
	
	public void setUsername(String username, String ip, int sessionNr) {
		this.username = username;
		logFile = new File("log"+System.getProperty("file.separator")+"session_"+sessionNr+".log");
		try {
			new File("log").mkdir();
			logFile.createNewFile();
			logOut = new FileWriter( logFile.getAbsolutePath() );
			
			writeHead(new SimpleDateFormat("'D.' dd.MM.yyyy ' kl.' HH:mm:ss").format(new Date()), ip, sessionNr, logOut);
			int direction;
			String text;
			String all;
			for (int i = 0 ; i < com.size() ; i++) {
				all = com.get(i).toString();
				direction=Integer.parseInt(all.charAt(0)+"");
				text = all.substring(1,all.length());
				write(text,direction);
			}
		}catch(IOException e) {
			bb.LogFileExcption(e,"Could not open the logFile " +  logFile.getAbsolutePath());
		}
	}
	
	public void stop() {
		if (logOut !=null) { //Makes the method work even if username was never set
			try {
				logOut.write("****************************************************************************************\n");
				logOut.write("\n");
				logOut.close();
				copyFile(username,logFile);
				}catch(IOException e) {
					bb.LogFileExcption(e,"Could not close the logFile " +  logFile.getAbsolutePath());
				}	
			}
	}
	
	// 1 = from client, 2 = from server
	public void write(String line, int direction) {
		if (logOut == null) {
			if (line.startsWith("PASS")) {
				String temp = "PASS ";
				for (int i = 0; i < line.length() - 5; i++) {
					temp += "*";
				}
				line = temp;
			}
			com.add(direction+line);
		}else {
			try {
				
				if (line.indexOf("/n") == -1) {
					switch (direction) {
						case 1:
							logOut.write("*C " + line + "\n");
							break;
						case 2:
							logOut.write("*S " + line + "\n");
					}
				}else {
					switch (direction) {
						case 1:
							writeManyLines("*C ", line);
							break;
						case 2:
							writeManyLines("*S ", line);
					}
				}				
			}catch(IOException e) {
				bb.LogFileExcption(e,"Could not write in the logFile " +  logFile.getAbsolutePath());
			}	
		}
	}
	
	private void writeManyLines(String front, String text) {
		try {
			StringTokenizer s = new StringTokenizer(text,"\n");
			while (s.hasMoreTokens()) {
				logOut.write(front + s.nextToken() + "\n");
			}
		}catch(IOException e) {
			bb.LogFileExcption(e,"Could not write in the logFile " +  logFile.getAbsolutePath());
		}	
		
		
	}
	
	//Metoden er static synchronized da der ikke m skrives af 2 klinter i den samme fil p n gang.
	//Det ville give en IOException, og noget log-info ville g tabt!
	private static synchronized void copyFile(String username, File logFile) {
		String fn = "log"+System.getProperty("file.separator")+username+".log";
    try {
			FileWriter output = new FileWriter(fn,true);
			FileReader input  = new FileReader(logFile);
			
			int size = 32768;//16384 
	    char[] b = new char[size];
			int read;
			
			while( true ){
			  read = input.read( b, 0, size );
			  if (read==-1)
			  	break;
			  output.write(b ,0, read);
			}
			input.close();
			output.close();
			logFile.delete();	
			LogFileReader.setLogFileUpdate(null, new File(fn));
    }catch(IOException e) {
	    bb.LogFileExcption(e,"Could not write the logFile " + logFile.getAbsolutePath() + " into the main logfile: " + fn);
    }
	}
}