package server.services;

import java.net.*;
import java.io.*;
import java.util.*;

import server.commands.*;
import server.event.*;
import setup.*;

public class Probe extends Thread{
  
  private Socket socket;
  private Dock dock;
  private Command com;
  private BigBrother bb = BigBrother.getInstance();
  public AbstractCommand login, commands;
	private boolean utf = false;

  public  Session ses;

  protected Probe(Socket socket, Dock dock, int sessionNr){
    this.socket = socket;
    this.dock = dock;
    ExtendedUserSession us = new ExtendedUserSession();
		ses = new Session(socket,us,this);
		ses.sessionNr = sessionNr;
	
    com = new Command(socket, ses);
		us.set(ses, com);
    new Welcome(com);
		resetChains();
  }

  protected void setUTF(boolean on) {
		utf = true;
  }
	
	public void addCommand(AbstractCommand aCom) {
		aCom.build(ses,com);
		aCom.addCommandLast(commands);
		commands=aCom.sort();
	}
	
	public void resetChains() {
		Chain C = new Chain(ses, com);
    commands = C.getDataChain();
    login = C.getLoginChain();	
	}
	
	public void run(){
    try {

			BufferedReader in = new BufferedReader( new InputStreamReader( socket.getInputStream() ) );
      String st="";
      
      while(st!=null){
				st = in.readLine();
				
        if (st!=null) {
            st = st.trim();
            doCommand(st);
        }
      }
      
    }catch(IOException e) {
			if (ses.logedIn)
				bb.probeException(e, "Could not read from sessionnumber: " + ses.sessionNr + ". The username was: " + ses.getLogin() + ".");
    }
		
		try{
	  	socket.close();
    }
    catch(Exception e){
      bb.probeException(e, "Cold not close the connection to sessionnumber: " + ses.sessionNr + ". The username was: " + ses.getLogin() + ".");
    }
		dock.disconnect(this);
		ses.us.userDisconnected(UserSessionObserver.USER_NORMAL_END);
  }
  
  private void doCommand(String command){
    ses.commandUpdate(command,1);
    
    StringTokenizer st = new StringTokenizer(command);
    String temp="";
		if (st.hasMoreTokens()) //If the String " " is send in a NullPointerException is thrown if no check is made
			temp = st.nextToken().toLowerCase();
    
		while(st.hasMoreTokens())
      temp+=" " + st.nextToken();

    if ( ses.logedIn )
      commands.action(temp);
    else
      login.action(temp);		
  }
}