package server.services;

import java.net.*;
import java.util.*;
import java.io.*;

import setup.*;
import server.*;
import server.event.*;
/**
 * The Session object holds all the information needed to make the FTP-Server work. 
 * It is a kind of information-holder, and service class for all of the Abstract Commands.
 */
public class Session {
  public int sessionNr=-1;
  public boolean logedIn = false;
  public boolean mode;	//TRUE = active mode, FALSE = passive mode.
	public Socket clientCommandSocket;
	public String serverIP=""; //Because internal and external users uses different ip's
  public String dataIP;
	public int dataPort;
	public XMLTag xml;
	public BigBrother bb = BigBrother.getInstance();
	public LogFileWriter logFile = new LogFileWriter();
	public FileHandler fileHandler;
	public FileSystem fileSystem;
	public ExtendedUserSession us;
	private Probe probe;
	
  protected Session(Socket clientCommandSocket, ExtendedUserSession eus, Probe probe){
		this.probe = probe;
		this.clientCommandSocket = clientCommandSocket;
		us = eus;
		String ip = Conf.getServerIP();  // hvis null, findes den ikke i XML-filen
		
    if (ip == null || ip.equals("") || clientCommandSocket.getInetAddress().getHostAddress().startsWith("192")){  //Udskift 192 med XML-muligheder
      try{
				ip = InetAddress.getLocalHost().getHostAddress();
			}catch(UnknownHostException uhe){
				bb.sessionException(uhe, "ses: Could not find the servers IP. Missing network?");
			}	
    }
		if (ip != null) {
			StringTokenizer str = new StringTokenizer(ip, ".");
			while (str.hasMoreTokens())
				serverIP += str.nextToken() + ",";
    }
	}
	
	//Her kommer al kommunikation fra/til klient ind. (incl. passwords).
	// 1 = from client, 2 = from server
  public void commandUpdate(String update, int direction){
		
		switch (direction) {
			case 1:
				us.commandUpdate(update);
				break;
			case 2:
				us.serverResponse(update);
		}
		
		if (!(logedIn && Conf.getUserOption(xml,"log").equals("no"))) {
			logFile.write(update, direction);
		}	
  }
	
	public void setAdmin() {
		probe.addCommand(new server.commands.Admin());
	}
	
	public void setLogedIn(boolean logedIn) {
		if (this.logedIn != logedIn) {
			if (Conf.getUserOption(xml,"log").equals("yes")) {
				if (logedIn) {
					logFile.setUsername(getLogin(), clientCommandSocket.getInetAddress().getHostAddress(), sessionNr);
				}else {
					logFile.stop();
					logFile = new LogFileWriter();
				}
			}
			if (logedIn) {
				fileSystem = new FileSystem(xml);
				us.userLogedIn();
			}else	 {
				us.userLogedOut();
				Dock.userLogedOut(this);
				probe.resetChains();
			}
		}
		this.logedIn = logedIn;
	}
	
	protected void bytesTransmitted(long[] arr) {
		us.bytesTransmitted(arr);
	}
	
	protected void fileTransferEnded(boolean successfull) {
		us.fileTransferEnded(successfull);
	}
	
	protected void fileTransferStarted(FileHandler fileHandler) {
		this.fileHandler = fileHandler;
		us.fileTransferStarted(fileHandler.getFile());
	}
	
	public void setClientIP() {
	}

	public String getLogin() {
		if (xml != null) {		
			try {
				return xml.getOptionValue("username");
			}catch(Exception e) {
				bb.sessionException(e, "ses: The sessionnumber: " + sessionNr + " has no 'username' deffined");
				return "";
			}
		}else
			return "";
	}
	
	// @see server.event.UserSessionObserver for reason code
	public void disconnectClient(int reasonCode) {
		us.userDisconnected(reasonCode);
		try {
			clientCommandSocket.close();
		}catch(IOException e) {
			bb.sessionException(e, "Could not disconnect sessionNr: " + sessionNr);
		}
	}
	
	public void setUTF(boolean on) {
		probe.setUTF(on);
  }
}