package setup;

import java.util.*;
import java.io.*;


/**
 * A XMLParser, is a structure that can read a XML file into the memory, so it can be used as a setup-file, or something like that.<br><br>
 * The root of the structure can be reached through the XMLParsers reference  to XMLTag called XML. If the structure
 * is modified at RunTime, it can be saved with the save() method. Comments will stay in the XML file by save(). <br>
 * Notice: There can occur reorganizing of the tags by save, because the XMLParser will rewrite the entire XML-file
 */
public class XMLParser{
  
  private LinkedList setup = new LinkedList();
  private BufferedReader in;
  /** The main tag describes the root of the XML-structure. This tag is not shown in the XML-document, but the comments and the option will be shown.*/
  public final XMLTag XML = new XMLTag("main");
  private File file;
  

  /**
   * Makes an XMLParser, and parses the passed file into its structure. 
   *
   *
   * @param   file  The file for the parser.
   * @exception   IOException  Thrown if the file cannot be opened.
   * @exception   NoSuchTagException  Thrown if there is an error in the XML structure in the file.
   */
  public XMLParser(File file) throws IOException, NoSuchTagException{
    this.file = file;
    readFile();
  }
  

  /**
   * Saves the current XML structure in the XML-File
   *
   * @exception   IOException  Thrown if the file cannot be saved!
   */
  public void save() throws IOException{
      PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(file)));
      String XMLContens = XML.getXMLString().toString();
      out.write(XMLContens.substring(0,XMLContens.length()-1));
      out.close();
  }
  

  /**
   * Reads the whole XML-File from the one end to the other in a while()-loop
   *
   * @exception   IOException  Thrown if the file cannot be opened!
   * @exception   NoSuchTagException  Thrown if there is an error in the XML structure in the file.
   */
  private void readFile() throws IOException, NoSuchTagException{
    in = new BufferedReader(new FileReader(file));
    while (in.ready() ){
      String line=in.readLine();
      if(line!=null)
        identifyLine(line);
    }
    in.close();
  }
    

  /**
   * Reads a line of the XML-File
   *
   * @return     the line.
   */
  private String getNextLine()throws IOException{
      return in.readLine();
  }
  

  /**
   * Identifies a line in the XML-File. There are 5 types of lines.<br>
   * 1 = endtag.<br>
   * 2 = starttag. &ltTAG&gt<br>
   * 3 = options. Has to be at least 2 words!<br>
   * 4 = kommentarer. Starts with the caracter # in column 1, or has a linelength at zero.<br>
   * 5 = other values (are ignored).<br>
   *
   * @param   line  
   * @exception   NoSuchTagException  
   */
  private void identifyLine(String line) throws NoSuchTagException{
    if ((!line.startsWith("#")) && (line.trim().length() != 0)){ //Skal linien slet ikke med i betragtning?
      String tag=line.trim();
      
        if (tag.startsWith("</") && tag.endsWith(">")){ //Er linien et sluttag?
          if (!XML.endTag(tag.substring(2,tag.length()-1)))
            throw new NoSuchTagException("Kunne ikke lukke tagget [" +  tag + "]");
        }else if (tag.startsWith("<") && tag.endsWith(">")){ //Er linien et starttag?
          XMLTag t = XML.addTagFromFile(tag.substring(1,tag.length()-1));
          if (t == null)
            throw new NoSuchTagException("Kunne ikke starte tagget [" +  tag + "]");
        }else{
          StringTokenizer st = new StringTokenizer(tag);
          if (st.countTokens()==1){
						String name = st.nextToken();
						String value="";
						XML.addOption(name,value);
        	}
					if (st.countTokens()>=2){
            String name = st.nextToken();
            String value=tag.substring(name.length()+1,tag.length());
            XML.addOption(name,value);
          }
        }
    }else{
      XML.addCommentFromFile(line);
    }
  }
}